unit Farbscala;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, ExtCtrls, ComCtrls, Windows, Controls, StdCtrls, Dialogs, Math,

 Direct3D9;  // Delete this, if yout don't use DirectX (It's just to import the modified FloatToStr-Functions)

const
 FarbScalaInfBlur : Byte = 1;
 FarbScalaInfAlpha : Byte = 2;
 FarbScalaInf0Transp : Byte = 4;


type
 TFarbScalaPoint = packed record
  S : Single;
  C : Cardinal;
 end;
 TFarbScala = record
  Inf : Byte;
  Min : Single;
  Max : Single;
  Table : Array of TFarbScalaPoint;
 end;
 PFarbScala = ^TFarbScala;

 TFarbScalaTool = class(TForm)
  private
   Panel           : TPanel;
   ButSave         : TButton;
   ButOpen         : TButton;
   ButAbort        : TButton;
   ButOK           : TButton;
   ButNew          : TButton;
   EditMin         : TEdit;
   LabelMin        : TLabel;
   EditMax         : TEdit;
   LabelMax        : TLabel;
   ScrollBox       : TScrollBox;
   CheckBoxAlpha   : TCheckBox;
   CheckBox0Transp : TCheckBox;
   CheckBoxBlur    : TCheckBox;
   LabelCheck      : TLabel;

   Scala       : TFarbScala;
   Saved       : Boolean;
   _Table      : Array of TFarbScalaPoint;
   TargetScala : PFarbScala;
   procedure ButAbortClick(Sender : TObject);
   procedure ButOKClick(Sender : TObject);
   procedure AddPointToTable(Sender : TObject);
   procedure ShowCurrentTable;
   procedure GetScala;
   procedure TablePointEditChange(Sender : TObject);
   procedure TablePointColorChange(Sender : TObject);
   procedure TablePointRemove(Sender : TObject);
   procedure PanelPaint(Sender : TObject);
   procedure EditMinMaxChange(Sender : TObject);
   procedure OpenFarbScala(Sender : TObject);
   procedure SaveFarbScala(Sender : TObject);
   procedure WriteFullAlphaEnabled(B : Boolean);
   procedure WriteNullTranspEnabled(B : Boolean);
   procedure WriteBlurEnabled(B : Boolean);
   function ReadFullAlphaEnabled : Boolean;
   function ReadNullTranspEnabled : Boolean;
   function ReadBlurEnabled : Boolean;
   procedure CheckBoxAlphaChange(Sender : TObject);
   procedure CheckBox0TranspChange(Sender : TObject);
   procedure CheckBoxBlurChange(Sender : TObject);
  protected
   procedure DoClose(var CloseAction: TCloseAction); override;
  public
   ForceSave : Boolean;
   ScalaPath : AnsiString;
   constructor Create(Sender : TComponent; FarbScala : PFarbScala);
   procedure LoadFromFile(FileName : AnsiString);
   procedure SaveToFile(FileName : AnsiString);
   property FullAlphaEnabled : Boolean read ReadFullAlphaEnabled write WriteFullAlphaEnabled;
   property NullTranspEnabled : Boolean read ReadNullTranspEnabled write WriteNullTranspEnabled;
   property BlurEnabled : Boolean read ReadBlurEnabled write WriteBlurEnabled;
 end;

procedure PaintFarbscalaToPanel(Panel: TPanel; FarbScala : PFarbScala);
function SaveFarbScalaToStream(Stream : TStream; FarbScala : PFarbScala) : Boolean;
function LoadFarbScalaFromStream(Stream : TStream; FarbScala : PFarbScala) : Boolean;


var
 _Farbscala : TFarbscala;

implementation

type TPaintFarbscalaToPanel = record
 i,j : integer;
 S   : Single;
 C   : Array [0..3] of Byte;
end;
var _PaintFarbscalaToPanel : TPaintFarbscalaToPanel;
procedure PaintFarbscalaToPanel(Panel: TPanel; FarbScala : PFarbScala);
begin with (_PaintFarbscalaToPanel) do begin
 if (Length(FarbScala^.Table) = 0) or (FarbScala^.Min >= FarbScala^.Max) then exit;
 i := 0;
 while (i < Panel.Width) do begin
  S := i / Panel.Width * (FarbScala^.Max - FarbScala^.Min) + FarbScala^.Min;
  j := Round(Abs(i / Panel.Width * (Length(FarbScala^.Table)-1)));
  if (j >= Length(FarbScala^.Table)) then j := Length(FarbScala^.Table) - 1;
  while (j > 0) and (FarbScala^.Table[j].S > S) do dec(j);
  while (j < Length(FarbScala^.Table) - 1) and (FarbScala^.Table[j+1].S < S) do inc(j);
  if (j = Length(FarbScala^.Table) - 1) then Panel.Canvas.Pen.Color := FarbScala^.Table[j].C
  else if (j = 0) and (FarbScala^.Table[j].S > S) then Panel.Canvas.Pen.Color := FarbScala^.Table[j].C
  else begin
   if Boolean(FarbScala^.Inf and FarbScalaInfBlur) then S := (S - FarbScala^.Table[j].S) / (FarbScala^.Table[j+1].S - FarbScala^.Table[j].S)
   else s := 0;
   C[0] := Round(PByte(@FarbScala^.Table[j].C + 0)^ * (1-S) + PByte(@FarbScala^.Table[j+1].C + 0)^ * (S));
   C[1] := Round(PByte(@FarbScala^.Table[j].C + 1)^ * (1-S) + PByte(@FarbScala^.Table[j+1].C + 1)^ * (S));
   C[2] := Round(PByte(@FarbScala^.Table[j].C + 2)^ * (1-S) + PByte(@FarbScala^.Table[j+1].C + 2)^ * (S));
   C[3] := 0;
   Panel.Canvas.Pen.Color := PCardinal(@C[0])^;
  end;
  Panel.Canvas.Line(i,0,i,Panel.Height);
  inc(i);
 end;
end; end;

function SaveFarbScalaToStream(Stream : TStream; FarbScala : PFarbScala) : Boolean;
var
 c : Cardinal;
 d : QWord;
begin
 Result := false;
 d := $5D81AF04;
 d := (d * d - SizeOf(TFarbScala.Inf)) mod $100000000;
 d := (d * d - SizeOf(TFarbScala.Min)) mod $100000000;
 d := (d * d - SizeOf(TFarbScala.Max)) mod $100000000;
 d := (d * d - SizeOf(TFarbScalaPoint)) mod $100000000;
 c := d;
 Stream.Write(c,4);
 Stream.Write(FarbScala^.Inf,SizeOf(TFarbScala.Inf));
 Stream.Write(FarbScala^.Min,SizeOf(TFarbScala.Min));
 Stream.Write(FarbScala^.Max,SizeOf(TFarbScala.Max));
 c := Length(FarbScala^.Table);
 Stream.Write(c,4);
 if (c > 0) then Stream.Write(FarbScala^.Table[0],c * SizeOf(TFarbScalaPoint));
 Result := true;
end;

function LoadFarbScalaFromStream(Stream : TStream; FarbScala : PFarbScala) : Boolean;
var
 c : Cardinal;
 d : QWord;
begin
 Result := false;
 Stream.Read(c,4);
 d := $5D81AF04;
 d := (d * d - SizeOf(TFarbScala.Inf)) mod $100000000;
 d := (d * d - SizeOf(TFarbScala.Min)) mod $100000000;
 d := (d * d - SizeOf(TFarbScala.Max)) mod $100000000;
 d := (d * d - SizeOf(TFarbScalaPoint)) mod $100000000;
 if (c <> d) then exit;
 Stream.Read(FarbScala^.Inf,SizeOf(TFarbScala.Inf));
 Stream.Read(FarbScala^.Min,SizeOf(TFarbScala.Min));
 Stream.Read(FarbScala^.Max,SizeOf(TFarbScala.Max));
 Stream.Read(c,4);
 SetLength(FarbScala^.Table,c);
 if (c > 0) then Stream.Read(FarbScala^.Table[0],c * SizeOf(TFarbScalaPoint));
 Result := true;
end;






constructor TFarbscalaTool.Create(Sender : TComponent; FarbScala : PFarbScala);
begin
 inherited Create(Sender);
 Saved := true;
 ScalaPath := '';
 ForceSave := false;
 BorderStyle := bsToolWindow;
 Width := 600;
 Height := 400;
 Top := (Screen.Height - Height) div 2;
 Left := (Screen.Width - Width) div 2;
 Caption := 'Farbscala Tool';
 TargetScala := FarbScala;
 Scala := FarbScala^;
 _Table := Scala.Table;

 ButOpen := TButton.Create(self);
 ButOpen.Parent := self;
 ButOpen.Left := 10;
 ButOpen.Top := 10;
 ButOpen.Caption := 'Öffnen';
 ButOpen.OnClick := @OpenFarbScala;

 ButSave := TButton.Create(self);
 ButSave.Parent := self;
 ButSave.AnchorToNeighbour(akLeft,10,ButOpen);
 ButSave.Top := 10;
 ButSave.Caption := 'Speichern';
 ButSave.OnClick := @SaveFarbScala;

 Panel := TPanel.Create(self);
 Panel.Parent := self;
 Panel.Left := 10;
 Panel.AnchorToNeighbour(akTop,10,ButOpen);
 Panel.AnchorParallel(akRight,10,self);
 Panel.Height := 35;
 Panel.OnPaint := @PanelPaint;

 EditMin := TEdit.Create(self);
 EditMin.Parent := self;
 EditMin.AnchorToNeighbour(akTop,10,Panel);
 EditMin.AnchorParallel(akLeft,10,self);
 EditMin.Width := 50;
 EditMin.Text := FloatToStr(Scala.Min);
 EditMin.OnChange := @EditMinMaxChange;

 LabelMin := TLabel.Create(self);
 LabelMin.Parent := self;
 LabelMin.AnchorToNeighbour(akTop,12,Panel);
 LabelMin.AnchorToNeighbour(akLeft,10,EditMin);
 LabelMin.Autosize := true;
 LabelMin.Caption := 'Min';

 EditMax := TEdit.Create(self);
 EditMax.Parent := self;
 EditMax.AnchorToNeighbour(akTop,10,Panel);
 EditMax.AnchorParallel(akRight,10,self);
 EditMax.Anchors := [akTop,akRight];
 EditMax.Width := 50;
 EditMax.Text := FloatToStr(Scala.Max);
 EditMax.OnChange := @EditMinMaxChange;

 LabelMax := TLabel.Create(self);
 LabelMax.Parent := self;
 LabelMax.AnchorToNeighbour(akTop,12,Panel);
 LabelMax.AnchorToNeighbour(akRight,10,EditMax);
 LabelMax.Anchors := [akTop,akRight];
 LabelMax.AutoSize := true;
 LabelMax.Caption := 'Max';

 ButNew := TButton.Create(self);
 ButNew.Parent := self;
 ButNew.AnchorToNeighbour(akTop,10,Panel);
 ButNew.Anchors := [akTop];
 ButNew.Width := EditMax.Height;
 ButNew.Height := EditMax.Height;
 ButNew.Left := (Width - ButNew.Width) div 2;
 ButNew.Caption := '+';
 ButNew.OnClick := @AddPointToTable;

 ButAbort := TButton.Create(Self);
 ButAbort.Parent := self;
 ButAbort.Anchors := [akRight,akBottom];
 ButAbort.AnchorParallel(akRight,10,self);
 ButAbort.AnchorParallel(akBottom,10,self);
 ButAbort.Width := 100;
 ButAbort.Caption := 'Abbrechen';
 ButAbort.OnClick := @ButAbortClick;

 ButOK := TButton.Create(self);
 ButOK.Parent := self;
 ButOK.Anchors := [akRight,akBottom];
 ButOK.AnchorToNeighbour(akRight,10,ButAbort);
 ButOK.AnchorParallel(akBottom,10,self);
 ButOK.Caption := 'OK';
 ButOK.OnClick := @ButOKClick;

 ScrollBox := TScrollBox.Create(self);
 ScrollBox.Parent := self;
 ScrollBox.Left := 10;
 ScrollBox.AnchorToNeighbour(akTop,10,ButNew);
 ScrollBox.AnchorParallel(akBottom,80,self);
 ScrollBox.AnchorParallel(akRight,10,self);

 LabelCheck := TLabel.Create(self);
 LabelCheck.Parent := self;
 LabelCheck.Anchors := [akLeft,akBottom];
 LabelCheck.Left := 0;
 LabelCheck.AnchorToNeighbour(akBottom,10,ButOK);
 LabelCheck.Caption := ' ';

 CheckBoxAlpha := TCheckBox.Create(self);
 CheckBoxAlpha.Parent := self;
 CheckBoxAlpha.Left := 10;
 CheckBoxAlpha.Anchors := [akLeft,akBottom];
 CheckBoxAlpha.AnchorToNeighbour(akLeft,10,LabelCheck);
 CheckBoxAlpha.AnchorToNeighbour(akBottom,10,ButOK);
 CheckBoxAlpha.Caption := 'Alphablending';
 CheckBoxAlpha.OnChange := @CheckBoxAlphaChange;

 CheckBox0Transp := TCheckBox.Create(self);
 CheckBox0Transp.Parent := self;
 CheckBox0Transp.Anchors := [akLeft,akBottom];
 CheckBox0Transp.AnchorToNeighbour(akLeft,20,CheckBoxAlpha);
 CheckBox0Transp.AnchorToNeighbour(akBottom,10,ButOK);
 CheckBox0Transp.Caption := 'Null ist Transparent';
 CheckBox0Transp.OnChange := @CheckBox0TranspChange;

 CheckBoxBlur := TCheckBox.Create(self);
 CheckBoxBlur.Parent := self;
 CheckBoxBlur.Anchors := [akLeft,akBottom];
 CheckBoxBlur.AnchorToNeighbour(akLeft,20,CheckBox0Transp);
 CheckBoxBlur.AnchorToNeighbour(akBottom,10,ButOK);
 CheckBoxBlur.Caption := 'Fließender Farbübergang';
 CheckBoxBlur.Checked := true;
 CheckBoxBlur.OnChange := @CheckBoxBlurChange;

 ShowCurrentTable;
 Show;
end;

procedure TFarbScalaTool.ShowCurrentTable;
var
  i : integer;
  Q : TPanel;
  P : TPanel;
  E : TEdit;
  C : TColorButton;
  B : TButton;
  T : TTrackBar;
  L : TLabel;
begin
 ScrollBox.DestroyComponents;
 Q := nil;
 for i := 0 to Length(_Table) - 1 do begin
  P := TPanel.Create(ScrollBox);
  P.Parent := ScrollBox;
  if (Q <> nil) then P.AnchorToNeighbour(akTop,0,Q);
  P.AnchorParallel(akRight,0,ScrollBox);
  P.Height := 50;
  Q := P;
  E := TEdit.Create(P);
  E.Parent := P;
  E.Top := 10;
  E.Left := 10;
  E.Width := 60;
  E.Name := 'S';
  E.Text := FloatToStr(_Table[i].S);
  if (isNaN(_Table[i].S)) then begin E.Text := ''; E.Color := $A0A0FF; end;
  E.OnChange := @TablePointEditChange;
  C := TColorButton.Create(P);
  C.Parent := P;
  C.Top := 10;
  C.AnchorToNeighbour(akLeft,20,E);
  C.Width := 40;
  C.ButtonColor := _Table[i].C;
  C.Name := 'C';
  C.OnColorChanged := @TablePointColorChange;
  T := TTrackBar.Create(P);
  T.Parent := P;
  T.Top := 10;
  T.AnchorToNeighbour(akLeft,10,C);
  T.Min := 0;
  T.Max := 255;
  T.Width := 255;
  T.TickMarks := tmBoth;
  T.TickStyle := tsNone;
  T.Height := 20;
  T.Position := PByte(@_Table[i].C+3)^;
  T.Visible := CheckBoxAlpha.Visible;
  T.Enabled := CheckBoxAlpha.Checked;
  T.OnChange := @TablePointColorChange;
  T.Name := 'T';
  L := TLabel.Create(P);
  L.Parent := P;
  L.AnchorToNeighbour(akTop,0,T);
  L.AnchorHorizontalCenterTo(T);
  L.Caption := 'Alpha';
  L.Visible := T.Visible;
  B := TButton.Create(ScrollBox);
  B.Parent := P;
  B.Top := 10;
  if (T.Visible) then B.AnchorToNeighbour(akLeft,30,T)
  else B.AnchorToNeighbour(akLeft,30,C);
  B.Width := 30;
  B.Caption := '-';
  B.OnClick := @TablePointRemove;
 end;
 Panel.Repaint;
 EditMin.Text := FloatToStr(Scala.Min);
 EditMax.Text := FloatToStr(Scala.Max);
 CheckBoxAlpha.OnChange := nil;
 CheckBoxAlpha.Checked := Boolean(Scala.Inf and FarbScalaInfAlpha);
 CheckBoxAlpha.OnChange := @CheckBoxAlphaChange;
 CheckBox0Transp.Checked := Boolean(Scala.Inf and FarbScalaInf0Transp);
 CheckBoxBlur.Checked := Boolean(Scala.Inf and FarbScalaInfBlur);
end;

procedure TFarbScalaTool.GetScala;
var
 i : integer;
 j : integer;
 P : TFarbScalaPoint;
 b : boolean;
begin
 i := ScrollBox.ControlCount;
 SetLength(_Table,i);
 for i := i-1 downto 0 do with (TPanel(ScrollBox.Controls[i])) do for j := 0 to ControlCount - 1 do begin
  if (Controls[j].Name = 'S') then begin
   try
    _Table[i].S := StrToFloat(PChar(TEdit(Controls[j]).Text));
   except
    _Table[i].S := NaN;
   end;
  end;
  if (Controls[j].Name = 'C') then begin
   PByte(@_Table[i].C+0)^ := PByte(@TColorButton(Controls[j]).ButtonColor+0)^;
   PByte(@_Table[i].C+1)^ := PByte(@TColorButton(Controls[j]).ButtonColor+1)^;
   PByte(@_Table[i].C+2)^ := PByte(@TColorButton(Controls[j]).ButtonColor+2)^;
  end;
  if (Controls[j].Name = 'T') then begin
   PByte(@_Table[i].C+3)^ := TTrackBar(Controls[j]).Position;
   if (not FullAlphaEnabled) then PByte(@_Table[i].C+3)^ := 0;
  end;
 end;
 Scala.Table := _Table;
 repeat begin
  b := true;
  for i := 0 to Length(Scala.Table) - 2 do if ((not IsNaN(Scala.Table[i+1].S)) and (IsNaN(Scala.Table[i].S) or (Scala.Table[i].S > Scala.Table[i+1].S))) then begin
   b := false;
   P := Scala.Table[i];
   Scala.Table[i] := Scala.Table[i+1];
   Scala.Table[i+1] := P;
  end;
 end until (b);
 for i := Length(Scala.Table) - 1 downto 0 do if (IsNaN(Scala.Table[i].S)) then SetLength(Scala.Table,i);
 Panel.Repaint;
 Saved := false;
end;

procedure TFarbScalaTool.TablePointEditChange(Sender : TObject);
var
 S     : Single;
 i,j   : integer;
 b     : Boolean;
 P     : TFarbScalaPoint;
 S1,S2 : AnsiString;
begin
 try
  S := StrToFloat(TEdit(Sender).Text);
  TEdit(Sender).Color := $FFFFFF;
 except
  TEdit(Sender).Color := $A0A0FF;
 end;
 GetScala;
end;

procedure TFarbScalaTool.TablePointColorChange(Sender : TObject);
begin
 GetScala;
end;

procedure TFarbScalaTool.TablePointRemove(Sender : TObject);
var
 P : TPanel;
 i : integer;
begin
 if not (TButton(Sender).Parent is TPanel) then exit;
 P := TPanel(TButton(Sender).Parent);
 if not (P.Owner is TScrollBox) then exit;
 for i := 0 to TScrollBox(P.Owner).ControlCount - 1 do if (TScrollBox(P.Owner).Controls[i] is TPanel) then begin
  if (TScrollBox(P.Owner).Controls[i].AnchorSideTop.Control = P) then TScrollBox(P.Owner).Controls[i].AnchorSideTop.Control := P.AnchorSideTop.Control;
 end;
 P.Free;
 GetScala;
end;

procedure TFarbScalaTool.AddPointToTable(Sender : TObject);
var i : integer;
begin
 i := Length(_Table);
 SetLength(_Table,i+1);
 _Table[i].S := NaN;
 _Table[i].C := 0;
 ShowCurrentTable;
end;

procedure TFarbScalaTool.EditMinMaxChange(Sender : TObject);
var S : Single;
begin
 try
  S := StrToFloat(TEdit(Sender).Text);
  TEdit(Sender).Color := $FFFFFF;
 except
  TEdit(Sender).Color := $A0A0FF;
  exit;
 end;
 if (Sender = EditMax) then Scala.Max := S
 else Scala.Min := S;
 Panel.Repaint;
end;

procedure TFarbScalaTool.PanelPaint(Sender : TObject);
begin
 PaintFarbscalaToPanel(TPanel(Sender),@Scala);
end;

procedure TFarbScalaTool.ButAbortClick(Sender : TObject);
begin
 Saved := true;
 PostMessage(Handle,WM_CLOSE,0,0);
end;

procedure TFarbScalaTool.ButOKClick(Sender : TObject);
begin
 TargetScala^ := Scala;
 PostMessage(Handle,WM_CLOSE,0,0);
end;

procedure TFarbScalaTool.OpenFarbScala(Sender : TObject);
var dlg : TOpenDialog;
begin
 dlg := TOpenDialog.Create(self);
 try
  dlg.Filter := 'Farbscala-Dateien|*.farbscala';
  if (ScalaPath <> '') then dlg.InitialDir := ExtractFilePath(ScalaPath);
  if (not dlg.Execute) then exit;
  if (not FileExists(dlg.FileName)) then exit;
  LoadFromFile(dlg.FileName);
 finally
  dlg.Free;
 end;
end;

procedure TFarbScalaTool.SaveFarbScala(Sender : TObject);
var dlg : TSaveDialog;
begin
 dlg := TSaveDialog.Create(self);
 try
  dlg.Filter := 'Farbscala-Dateien|*.farbscala';
  if (ScalaPath <> '') then dlg.InitialDir := ExtractFilePath(ScalaPath);
  if (not dlg.Execute) then exit;
  if (Length(dlg.FileName) < 11) then exit;
  if (FileExists(dlg.FileName)) then if (MessageDlg('Datei Existiert bereits. Überschreiben?',MtCustom, [MbYes, MbNo], 0) = mrNo) then exit;
  SaveToFile(dlg.FileName);
 finally
  dlg.Free;
 end;
end;

procedure TFarbScalaTool.SaveToFile(FileName : AnsiString);
var f : TFileStream;
begin
 f := TFileStream.Create(FileName,fmCreate or fmOpenWrite);
 try
  if (not FullAlphaEnabled) then Scala.Inf := Scala.Inf and (not FarbScalaInfAlpha);
  if (not NullTranspEnabled) then Scala.Inf := Scala.Inf and (not FarbScalaInf0Transp);
  if (not BlurEnabled) then Scala.Inf := Scala.Inf and (not FarbScalaInfBlur);
  SaveFarbScalaToStream(f,@Scala);
 finally
  f.Free;
 end;
 ScalaPath := FileName;
 Saved := true;
end;

procedure TFarbScalaTool.LoadFromFile(FileName : AnsiString);
var
 f : TFileStream;
 i : integer;
 S : Single;
begin
 if (ForceSave) and (not Saved) then if (MessageDlg(
   'Die Änderungen an der Farbscala wurden nicht gespeichert. Möchten Sie die Änderungen speichern?',
   MtCustom, [MbYes, MbNo], 0) = mrYes) then SaveFarbScala(nil);
 f := TFileStream.Create(FileName,fmOpenRead);
 try
  if (not LoadFarbScalaFromStream(f,@Scala)) then begin
   ShowMessage('Fehler beim Laden der Farbscala. Möglicher Weise handelt es sich nicht um eine gültige Farbscala-Datei oder diese wurde von einer anderen Version des Tools erstellt.');
   Scala.Inf := 0;
   Scala.Min := 0;
   Scala.Max := 0;
   SetLength(Scala.Table,0);
  end;
 finally
  f.Free;
 end;
 ScalaPath := FileName;
 _Table := Scala.Table;
 ShowCurrentTable;
end;

procedure TFarbScalaTool.WriteFullAlphaEnabled(B : Boolean);
begin
 CheckBoxAlpha.Visible := B;
 if (B) then CheckBox0Transp.AnchorSideLeft.Control := CheckBoxAlpha
 else CheckBox0Transp.AnchorSideLeft.Control := LabelCheck;
end;

procedure TFarbScalaTool.WriteNullTranspEnabled(B : Boolean);
begin
 CheckBox0Transp.Visible := B;
 if (B) then CheckBoxBlur.AnchorSideLeft.Control := CheckBox0Transp
 else CheckBoxBlur.AnchorSideLeft.Control := CheckBox0Transp.AnchorSideLeft.Control;
end;

procedure TFarbScalaTool.WriteBlurEnabled(B : Boolean);
begin
 CheckBoxBlur.Visible := B;
end;

function TFarbScalaTool.ReadFullAlphaEnabled : Boolean;
begin
 Result := CheckBoxAlpha.Visible;
end;

function TFarbScalaTool.ReadNullTranspEnabled : Boolean;
begin
 Result := CheckBox0Transp.Visible;
end;

function TFarbScalaTool.ReadBlurEnabled : Boolean;
begin
 Result := CheckBoxBlur.Visible;
end;

procedure TFarbScalaTool.CheckBoxAlphaChange(Sender : TObject);
begin
 if (TCheckBox(Sender).Checked) then Scala.Inf := Scala.Inf or FarbScalaInfAlpha
 else Scala.Inf := Scala.Inf and (not FarbScalaInfAlpha);
 ShowCurrentTable;
end;

procedure TFarbScalaTool.CheckBox0TranspChange(Sender : TObject);
begin
 if (TCheckBox(Sender).Checked) then Scala.Inf := Scala.Inf or FarbScalaInf0Transp
 else Scala.Inf := Scala.Inf and (not FarbScalaInf0Transp);
end;

procedure TFarbScalaTool.CheckBoxBlurChange(Sender : TObject);
begin
 if (TCheckBox(Sender).Checked) then Scala.Inf := Scala.Inf or FarbScalaInfBlur
 else Scala.Inf := Scala.Inf and (not FarbScalaInfBlur);
 Panel.Repaint;
end;

procedure TFarbScalaTool.DoClose(var CloseAction : TCloseAction);
begin
 if (ForceSave) and (not Saved) then if (MessageDlg(
  'Die Änderungen an der Farbscala wurden nicht gespeichert und werden mit dem nächsten Start des Programmmes verloren gehen. Möchten Sie die Änderungen speichern?',
  MtCustom, [MbYes, MbNo], 0) = mrYes) then SaveFarbScala(nil);
 inherited;
end;

end.

