{
 Diese Unit stellt die Klasse TGraph2D zur Verfügung. Dies ist eine Erweiterung
 von TPanel, welche in der Lage ist, Linien und verschiebbare Punkte in einem
 Koordinatensystem darzustellen.
}



unit Graph2D;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, StdCtrls, ExtCtrls, Math, Controls, Windows, Forms, Graphics;

type
 TGraph2D = class;
 TGraph2DPoint = packed record
  X : Double;
  Y : Double;
 end;
 TGraph2DLine = Array of TGraph2DPoint;
 TGraph2D = class(TPanel)
  private
   ViewRX1 : Double;
   ViewRY1 : Double;
   ViewRX2 : Double;
   ViewRY2 : Double;
   EditFocus : TEdit;

   KeyR    : Boolean;
   KeyL    : Boolean;
   KeyT    : Boolean;
   KeyB    : Boolean;
   ShiftS  : TShiftState;
   Timer   : TTimer;

   Move    : Boolean;
   StartC  : TPoint;
   StartPX : Double;
   StartPY : Double;
   procedure DrawAxis;
   procedure DrawLines;
   procedure MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
   procedure MouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
   procedure MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
   procedure MouseWheel(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
   procedure TimerEV(Sender : TObject);
   procedure KeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
   procedure KeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
  public
   Lines      : Array of TGraph2DLine;
   OnMove     : TNotifyEvent;
   MinXRange  : double;
   MaxXRange  : double;
   MinYRange  : double;
   MaxYRange  : double;
   MinX       : double;
   MaxX       : double;
   MaxY       : double;
   MinY       : double;
   Speed      : word;
   PixDiffPerLine : word;
   constructor Create(_Owner : TComponent);
   procedure SetViewArea(X1,Y1,X2,Y2 : Double);
   function XPosToScreen(Pos : Double) : integer;
   function YPosToScreen(Pos : Double) : integer;
   function XScreenToPos(Screen : integer) : Double;
   function YScreenToPos(Screen : integer) : Double;
   procedure DoPaint(Sender : TObject);
   destructor Destroy; override;
 end;

implementation

constructor TGraph2D.Create(_Owner : TComponent);
begin
 inherited Create(_Owner);
 SetViewArea(-5,5,5,-5);
 Move := false;
 KeyL := false;
 KeyR := false;
 KeyT := false;
 KeyB := false;
 ShiftS := [];
 Timer := TTimer.Create(self);
 Timer.Interval := 40;
 Timer.Enabled := false;
 Timer.OnTimer := @TimerEv;
 EditFocus := TEdit.Create(Self);
 EditFocus.Top := -50;
 EditFocus.Parent := self;
 OnMove := nil;
 OnMouseDown := @MouseDown;
 OnMouseUp := @MouseUp;
 OnMouseMove := @MouseMove;
 OnMouseWheel := @MouseWheel;
 OnKeyDown := @KeyDown;
 OnKeyUp := @KeyUp;
 OnPaint := @DoPaint;
 EditFocus.OnKeyDown := @KeyDown;
 EditFocus.OnKeyUp := @KeyUp;
 BorderStyle := bsNone;
 MinXRange := 0.0001;
 MaxXRange := 65535;
 MinYRange := 0.0001;
 MaxYRange := 1000000000;
 MinX := -100000000000;
 MaxX :=  100000000000;
 MaxY :=  100000000000;
 MinY := -100000000000;
 PixDiffPerLine := 50;
 Speed := 10;
end;

procedure TGraph2D.SetViewArea(X1,Y1,X2,Y2 : Double);
var
 Cash : Double;
begin
 if (X2 < X1) then begin Cash := X1; X1 := X2; X2 := Cash; end;
 if (Y2 > Y1) then begin Cash := Y1; Y1 := Y2; Y2 := Cash; end;
 if (X2 - X1 < MinXRange) then X2 := X1 + MinXRange;
 if (Y1 - Y2 < MinYRange) then Y2 := Y1 - MinYRange;
 if (X2 - X1 > MaxXRange) then X2 := X1 + MaxXRange;
 if (Y1 - Y2 > MaxYRange) then Y2 := Y1 - MaxYRange;
 if (X2 > MaxX) then begin X2 := MaxX; X1 := X2 - ViewRX2 + ViewRX1; end;
 if (X1 < MinX) then begin X1 := MinX; X2 := X1 + ViewRX2 - ViewRX1; end;
 if (Y1 > MaxY) then begin Y1 := MaxY; Y2 := Y1 - ViewRY1 + ViewRY2; end;
 if (Y2 < MinY) then begin Y2 := MinY; Y1 := Y2 + ViewRY1 - ViewRY2; end;
 ViewRX1 := X1;
 ViewRY1 := Y1;
 ViewRX2 := X2;
 ViewRY2 := Y2;
 if (OnMove <> nil) then OnMove(self);
end;

procedure TGraph2D.MouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 EditFocus.SetFocus;
 StartC.X := X;
 StartC.Y := Y;
 StartPX := ViewRX1;
 StartPY := ViewRY1;
 Move := true;
end;

procedure TGraph2D.MouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 Move := false;
end;

procedure TGraph2D.MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var d,e : Double;
begin
 if (Move) then begin
  d := StartPX + (StartC.X - X) / Width * (ViewRX2 - ViewRX1);
  e := StartPY + (StartC.Y - Y) / Height * (ViewRY2 - ViewRY1);
  SetViewArea(d,e,d + ViewRX2 - ViewRX1,e + ViewRY2 - ViewRY1);
  Self.DoPaint(Sender);
 end;
end;

procedure TGraph2D.MouseWheel(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
var
 Sx,Sy                   : Single;
 WheelDeltaX,WheelDeltaY : integer;
begin
 if (Width <= 0) then exit;
 if (Height <= 0) then exit;
 WheelDeltaX := Round(WheelDelta / 1000 * Width);
 WheelDeltaY := Round(WheelDelta / 1000 * Height);
 Sx := MousePos.X / Width;
 Sy := MousePos.Y / Height;
 SetViewArea(XScreenToPos(Round(WheelDeltaX * Sx)),YScreenToPos(Round(WheelDeltaY * Sy)),XScreenToPos(Width - Round(WheelDeltaX * (1 - Sx))),YScreenToPos(Height - Round(WheelDeltaY * (1 - Sy))));
 DoPaint(Sender);
end;

procedure TGraph2D.TimerEV(Sender : TObject);
var
 SX1,SY1,SX2,SY2 : shortint;
 X1,Y1,X2,Y2     : double;
 b               : boolean;
begin
 if (not KeyT) and (not KeyB) and (not KeyL) and (not KeyR) then begin Timer.Enabled := false; exit; end;
 SX1 := 0;
 SY1 := 0;
 SX2 := 0;
 SY2 := 0;
 b := false;
 if (KeyR) and (not KeyL) then begin SX1 := Speed; SX2 := Speed; end;
 if (KeyL) and (not KeyR) then begin SX1 := -Speed; SX2 := -Speed; end;
 if (KeyR) and (KeyL) and (ssCtrl in ShiftS) then begin SX1 := -Speed; SX2 := Speed; b := true; end;
 if (KeyR) and (KeyL) and (not (ssCtrl in ShiftS)) then begin SX1 := Speed; SX2 := -Speed; end;
 X1 := XScreenToPos(SX1);
 X2 := XScreenToPos(SX2 + Width);
 if (b) and (X1 < MinX) then X1 := MinX;
 if (b) and (X2 > MaxX) then X2 := MaxX;

 b := false;
 if (KeyB) and (not KeyT) then begin SY1 := Speed; SY2 := Speed; end;
 if (KeyT) and (not KeyB) then begin SY1 := -Speed; SY2 := -Speed; end;
 if (KeyT) and (KeyB) and (ssCtrl in ShiftS) then begin SY1 := -Speed; SY2 := Speed; b := true; end;
 if (KeyT) and (KeyB) and (not (ssCtrl in ShiftS)) then begin SY1 := Speed; SY2 := -Speed; end;
 Y1 := YScreenToPos(SY1);
 Y2 := YScreenToPos(SY2 + Height);
 if (b) and (Y2 < MinY) then Y2 := MinY;
 if (b) and (Y1 > MaxY) then Y1 := MaxY;

 SetViewArea(X1,Y1,X2,Y2);
 DoPaint(Sender);
end;

procedure TGraph2D.KeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
 if (Key = 37) then KeyL := true
 else if (Key = 38) then KeyT := true
 else if (Key = 39) then KeyR := true
 else if (Key = 40) then KeyB := true;
 ShiftS := Shift;
 Timer.Enabled := true;
end;

procedure TGraph2D.KeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
 if (Key = 37) then KeyL := false
 else if (Key = 38) then KeyT := false
 else if (Key = 39) then KeyR := false
 else if (Key = 40) then KeyB := false;
 ShiftS := Shift;
end;

function TGraph2D.XPosToScreen(Pos : Double) : integer;
var D : Double;
begin
 D := ((Pos - ViewRX1) / (ViewRX2 - ViewRX1)) * Width;
 if (D > 1000000) then Result := 1000000
 else if (D < -1000000) then Result := -1000000
 else Result := Round(D);
end;

function TGraph2D.YPosToScreen(Pos : Double) : integer;
var D : Double;
begin
 D := (Pos - ViewRY1) / (ViewRY2 - ViewRY1) * Height;
 if (D > 1000000) then Result := 1000000
 else if (D < -1000000) then Result := -1000000
 else Result := Round(D);
end;

function TGraph2D.XScreenToPos(Screen : integer) : Double;
begin
 Result := 0;
 if (Width <= 0) then exit;
 Result := ViewRX1 + (ViewRX2 - ViewRX1) * (Screen / Width);
end;

function TGraph2D.YScreenToPos(Screen : integer) : Double;
begin
 Result := 0;
 if (Height <= 0) then exit;
 Result := ViewRY1 + (ViewRY2 - ViewRY1) * (Screen / Height);
end;

function IntPower(Basis : Double; Exp : Integer) : Double;
var i : integer;
begin
 Result := 1;
 for i := 0 to abs(Exp) - 1 do Result := Result * Basis;
 if (Exp < 0) then Result := 1 / Result;
end;

procedure GetPosOfRegularPoints(From,UpTo : Double; Count : Word; var Start,Space : Double);
var
 Exp : integer;
 K   : Byte;
 D   : Double;
function GetNextSpace : Double;
begin
 if (K = 5) then K := 2
 else if (K = 2) then K := 1
 else begin
  K := 5;
  dec(Exp);
 end;
 Result := K * IntPower(10,Exp);
end;
begin
 if (UpTo = From) then begin Start := From; Space := 0; exit; end;
 Exp := floor(Ln(Abs(UpTo - From)) / Ln(10)) + 1;
 K := 2;
 Space := K * IntPower(10,Exp);
 repeat begin
  D := GetNextSpace;
  if (D * Count >= UpTo - From) then begin
   Space := D;
   Start := ceil(From / Space) * Space;
  end;
 end until (D * Count < UpTo - From);
 // showmessage('From: ' + floattostr(From) + ' To: ' + floattostr(UpTo) + ' Count: ' + inttostr(Count) + ' Start: ' + floattostr(Start) + ' Space: ' + floattostr(Space));
end;

procedure TGraph2D.DrawAxis;
var
 Pos        : integer;
 LineLength : integer;
 i,j,k      : integer;
 d,e        : double;
begin
 Canvas.Pen.Width := 1;
 Canvas.Pen.Color := clBlack;
 Canvas.Font.Color := clBlack;
 Pos := XPosToScreen(0);
 if (Pos < 0) then Pos := 0;
 if (Pos >= Width) then Pos := Width - 1;
 k := Pos + 1;
 if (k + 20 > Width) then k := Width - 20;
 LineLength := Floor(Height / PixDiffPerLine);
 GetPosOfRegularPoints(ViewRY2,ViewRY1,LineLength,d,e);
 for i := 0 to LineLength - 1 do begin
  j := YPosToScreen(d + e*i);
  Canvas.TextOut(k,j - 1,FloatToStr(Round((d + e*i) * 100000000) / 100000000));
  Canvas.Line(Pos - 2,j,Pos + 2,j);
 end;
 Canvas.Line(Pos,0,Pos,Height);

 Pos := YPosToScreen(0);
 if (Pos < 0) then Pos := 0;
 if (Pos >= Height) then Pos := Height - 1;
 k := Pos + 1;
 if (k + 18 > Height) then k := Height - 18;
 LineLength := Floor(Width / PixDiffPerLine);
 GetPosOfRegularPoints(ViewRX1,ViewRX2,LineLength,d,e);
 for i := 0 to LineLength - 1 do begin
  j := XPosToScreen(d + e*i);
  Canvas.TextOut(j + 1,k,FloatToStr(Round((d + e*i))));
  Canvas.Line(j,Pos + 2,j,Pos - 2);
 end;
 Canvas.Line(0,Pos,Width,Pos);
end;

procedure TGraph2D.DrawLines;
var i,j : integer;
begin
 Canvas.Pen.Color := clBlack;
 for i := 0 to Length(Lines) - 1 do if (Length(Lines[i]) > 0) then begin
  Canvas.MoveTo(XPosToScreen(Lines[i][0].X),YPosToScreen(Lines[i][0].Y));
  for j := 1 to Length(Lines[i]) - 1 do if (Lines[i][j].X < 1) or (Lines[i][j].X > 65535) then Canvas.MoveTo(XPosToScreen(Lines[i][j].X),YPosToScreen(Lines[i][j].Y))
  else Canvas.LineTo(XPosToScreen(Lines[i][j].X),YPosToScreen(Lines[i][j].Y));
 end;
end;

procedure TGraph2D.DoPaint(Sender : TObject);
begin
 Canvas.Brush.Color := clSilver;
 Canvas.FillRect(0,0,Width,Height);
 DrawAxis;
 DrawLines;
end;

destructor TGraph2D.Destroy;
begin
 inherited;
end;

end.

