unit Settings;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FarbScala, Direct3D9, Windows, IniFiles;

type
 TSettings = record
  ShowType      : Byte;
  BlendoutInAng : Boolean;
  BlendoutA     : word;
  BlendoutB     : word;
  HFactor       : Single;
  BGColor       : cardinal;
  Farbscala     : AnsiString;
 end;
 TControlSettings = record
  KFullscreen : Byte;
  KForward    : Byte;
  KBack       : Byte;
  KLeft       : Byte;
  KRight      : Byte;
  KTop        : Byte;
  KBottom     : Byte;
  KRot1       : Byte;
  KRot2       : Byte;
  KCenter     : Byte;
 end;

var
 _Settings   : TSettings;
 SetDir      : AnsiString = '';
 SaveTimeout : Cardinal = 10000;
 ContSet     : TControlSettings;


procedure LoadSettings;
procedure SaveSettings;
procedure SaveSettingsNow;
procedure DefaultSettings;

implementation

var
 _SaveTimeout : Cardinal;
 Timer        : Pointer = nil;

type
 TTimerProc = procedure(_HWND : THandle; uMsg : UInt; idEvent : UInt; dwTime : Cardinal); stdcall;

procedure SaveSettingsNow;
var ini : TIniFile;
begin
 if (Timer <> nil) then KillTimer(0,PTRUInt(Timer));
 Timer := nil;
 ini := TIniFile.Create(SetDir + 'Settings.ini');
 try
  with (_Settings) do begin
   ini.WriteInteger('Anzeige','ShowType',ShowType);
   ini.WriteBool('Anzeige','BlendoutInAng',BlendoutInAng);
   ini.WriteInteger('Anzeige','BlendoutA',BlendoutA);
   ini.WriteInteger('Anzeige','BlendoutB',BlendoutB);
   ini.WriteFloat('Anzeige','HFactor',HFactor);
   ini.WriteString('Anzeige','Farbscala',Farbscala);
   ini.WriteInteger('Anzeige','BGColor',BGColor);
  end;
  with (ContSet) do begin
   ini.WriteInteger('Steuerung','Fullscreen',KFullscreen);
   ini.WriteInteger('Steuerung','Forward',KForward);
   ini.WriteInteger('Steuerung','Back',KBack);
   ini.WriteInteger('Steuerung','Left',KLeft);
   ini.WriteInteger('Steuerung','Right',KRight);
   ini.WriteInteger('Steuerung','Top',KTop);
   ini.WriteInteger('Steuerung','Bottom',KBottom);
   ini.WriteInteger('Steuerung','Rot1',KRot1);
   ini.WriteInteger('Steuerung','Rot2',KRot2);
   ini.WriteInteger('Steuerung','Center',KCenter);
  end;
 finally
  ini.Free;
 end;
end;

procedure TimerProc(_HWND : THandle; uMsg : Cardinal; idEvent : Pointer; dwTime : Cardinal); stdcall;
begin
 if (dwTime >= _SaveTimeout) then begin
  SaveSettingsNow;
 end;
end;

procedure LoadSettings;
var
 ini : TIniFile;
 f   : TFileStream;
begin
 ini := TIniFile.Create(SetDir + 'Settings.ini');
 try
  with (_Settings) do begin
   ShowType := ini.ReadInteger('Anzeige','ShowType',ShowType);
   BlendoutInAng := ini.ReadBool('Anzeige','BlendoutInAng',BlendoutInAng);
   BlendoutA := ini.ReadInteger('Anzeige','BlendoutA',BlendoutA);
   BlendoutB := ini.ReadInteger('Anzeige','BlendoutB',BlendoutB);
   HFactor := ini.ReadFloat('Anzeige','HFactor',HFactor);
   BGColor := ini.ReadInteger('Anzeige','BGColor',BGColor);
   Farbscala := ini.ReadString('Anzeige','Farbscala',Farbscala);
  end;
  with (ContSet) do begin
   ini.WriteInteger('Steuerung','Fullscreen',KFullscreen);
   ini.WriteInteger('Steuerung','Forward',KForward);
   ini.WriteInteger('Steuerung','Back',KBack);
   ini.WriteInteger('Steuerung','Left',KLeft);
   ini.WriteInteger('Steuerung','Right',KRight);
   ini.WriteInteger('Steuerung','Top',KTop);
   ini.WriteInteger('Steuerung','Bottom',KBottom);
   ini.WriteInteger('Steuerung','Rot1',KRot1);
   ini.WriteInteger('Steuerung','Rot2',KRot2);
   ini.WriteInteger('Steuerung','Center',KCenter);
  end;
 finally
  ini.Free;
 end;
 if (FileExists(_Settings.Farbscala)) then begin
  f := TFileStream.Create(_Settings.Farbscala,fmOpenRead);
  LoadFarbScalaFromStream(f,@_FarbScala);
  f.Free;
 end;
end;

procedure SaveSettings;
begin
 if (Timer = nil) then PTRUInt(Timer) := SetTimer(0, PTRUInt(Timer),SaveTimeout div 10 + 1,TTimerProc(@TimerProc));
 _SaveTimeout := GetTickCount + SaveTimeout;
end;

procedure DefaultSettings;
begin
 with (_Settings) do begin
  ShowType := 0;
  BlendoutInAng := false;
  BlendoutA := 0;
  BlendoutB := 0;
  HFactor := 1 / 500000;
  Farbscala := '';
  BGColor := 0;
  if (FileExists(SetDir + 'Schwarz-Weiss.farbscala')) then FarbScala := SetDir + 'Schwarz-Weiss.farbscala';
 end;
 with (ContSet) do begin
  KFullscreen := 122;
  KForward    := 38;
  KBack       := 40;
  KLeft       := 65;
  KRight      := 68;
  KTop        := 87;
  KBottom     := 83;
  KRot1       := 37;
  KRot2       := 39;
  KCenter     := 32;
 end;
 _FarbScala.Inf := FarbScalaInf0Transp or FarbScalaInfBlur;
 _FarbScala.Max := 0.2;
 _FarbScala.Min := -0.2;
 SetLength(_FarbScala.Table,2);
 _FarbScala.Table[0].S := _FarbScala.Min;
 _FarbScala.Table[0].C := 0;
 _FarbScala.Table[1].S := _FarbScala.Max;
 _FarbScala.Table[1].C := $FFFFFF;
end;

finalization

if (Timer <> nil) then begin
 KillTimer(0,PTRUInt(Timer));
 Timer := nil;
 SaveSettingsNow;
end;


end.

