unit UnitDisplay; 

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, StdCtrls, Graphics, Dialogs,
  ComCtrls, ExtCtrls, WaveSimulation, Farbscala, Windows, Settings, Direct3D9;

type

  { TFormDisplay }

  TFormDisplay = class(TForm)
    CheckBox1: TCheckBox;
    ColorButton1: TColorButton;
    ComboBox1: TComboBox;
    Edit1: TEdit;
    Edit2: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Panel1: TPanel;
    TrackBar1: TTrackBar;
   procedure BGColor(Sender: TObject);
   procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
   procedure FormCreate(Sender: TObject);
   procedure HFactorChange(Sender: TObject);
   procedure ShowFarbscala(Sender: TObject);
   procedure ShowType(Sender: TObject);
   procedure AngChange(Sender: TObject);
   procedure FarbScalaPaint(Sender : TObject);
   procedure FarbScalaClose(Sender : TObject; var CloseAction : TCloseAction);
   procedure FormShow(Sender: TObject);
  private
    ChangeEnabled : Boolean;
  public
    { public declarations }
  end; 

var
  FormDisplay: TFormDisplay;

implementation

{$R *.lfm}

var FarbScalaTool : TFarbScalaTool = nil;

procedure TFormDisplay.FormCreate(Sender: TObject);
begin
 ChangeEnabled := false;
 Panel1.OnPaint := @FarbScalaPaint;
end;

procedure TFormDisplay.FormShow(Sender: TObject);
begin with (_Settings) do begin
 ComboBox1.ItemIndex := ShowType;
 CheckBox1.Checked := BlendoutInAng;
 Edit1.Text := FloatToStr(BlendoutA / 65536 * 360);
 Edit2.Text := FloatToStr(BlendoutB / 65536 * 360);
 TrackBar1.Position := Round(sqrt(HFactor * 500000) * 50);
 ColorButton1.ButtonColor := _Settings.BGColor;
 ChangeEnabled := true;
end; end;

procedure TFormDisplay.ShowType(Sender: TObject);
begin
 if (not ChangeEnabled) then exit;
 _Settings.ShowType := TComboBox(Sender).ItemIndex;
 SaveSettings;
end;

procedure TFormDisplay.HFactorChange(Sender: TObject);
begin
 if (not ChangeEnabled) then exit;
 _Settings.HFactor := (TrackBar1.Position / 50) * (TrackBar1.Position / 50) / 500000;
 if (not Sim.IsWorking) then Sim.LoadToTex;
 SaveSettings;
end;

procedure TFormDisplay.FarbScalaPaint(Sender : TObject);
begin
 PaintFarbscalaToPanel(TPanel(Sender),@_FarbScala);
end;

procedure TFormDisplay.ShowFarbscala(Sender: TObject);
begin
 FarbScalaTool := TFarbScalaTool.Create(self,@_FarbScala);
 FarbScalaTool.ForceSave := true;
 if (_Settings.Farbscala = '') then FarbScalaTool.ScalaPath := SetDir + 'Schwarz-Weiss.farbscala'
 else FarbScalaTool.ScalaPath := _Settings.Farbscala;
 FarbScalaTool.FullAlphaEnabled := false;
 FarbScalaTool.OnClose := @FarbScalaClose;
end;

procedure TFormDisplay.FarbScalaClose(Sender : TObject; var CloseAction : TCloseAction);
begin
 if (not ChangeEnabled) then exit;
 if (FarbScalaTool.ScalaPath <> _Settings.Farbscala) then begin
  _Settings.Farbscala := FarbScalaTool.ScalaPath;
  SaveSettings;
 end;
 Panel1.Repaint;
 if (not Sim.IsWorking) then Sim.LoadToTex;
 CloseAction := caFree;
 FarbScalaTool := nil;
end;

procedure TFormDisplay.AngChange(Sender: TObject);
var i : integer;
begin
 if (not ChangeEnabled) then exit;
 if (CheckBox1.Checked) then begin
  try
   i := Round(StrToInt(Edit1.Text) / 360 * 65536);
   while (i < 0) do i := i + 65536;
   _Settings.BlendoutA := i;
   i := Round(StrToInt(Edit2.Text) / 360 * 65536);
   while (i < 0) do i := i + 65536;
   _Settings.BlendoutB := i;
  except
   _Settings.BlendoutA := 0;
   _Settings.BlendoutB := 0;
  end;
 end else begin
  _Settings.BlendoutA := 0;
  _Settings.BlendoutB := 0;
 end;
 SaveSettings;
end;

procedure TFormDisplay.BGColor(Sender: TObject);
begin
 if (not ChangeEnabled) then exit;
 _Settings.BGColor := TColorButton(Sender).ButtonColor;
 SaveSettings;
 Application.ProcessMessages;
end;

procedure TFormDisplay.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
 ChangeEnabled := false;
 if (FarbScalaTool <> nil) then FarbScalaTool.Free;
 FarbScalaTool := nil;
end;

end.

