﻿unit UnitMain; 

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls,
  StdCtrls, ComCtrls, WaveSimulation, d3dx9, direct3d9, UnitDisplay,
  UnitQuality, UnitSources, UnitControls, WaveSimulationTools, Windows,
  Farbscala, Settings, ShellAPI;

type

  { TFormSim }

  TFormSim = class(TForm)
    Button1: TButton;
    Button10: TButton;
    Button2: TButton;
    Button3: TButton;
    Button4: TButton;
    Button5: TButton;
    Button6: TButton;
    Button7: TButton;
    Button8: TButton;
    Button9: TButton;
    CheckBox1: TCheckBox;
    EditMove3D: TEdit;
    Label1: TLabel;
    Panel1: TPanel;
    Timer1: TTimer;
    procedure ButSteuerungClick(Sender: TObject);
    procedure Change3DSet(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Move3D;
    procedure Move3DKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure Move3DKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure onStop;
    procedure onStepReady;
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure Load(Sender: TObject);
    procedure PanelResize(Sender: TObject);
    procedure PlaceFourierGraph(Sender: TObject);
    procedure PlaceGraph2D(Sender: TObject);
    procedure PlaceVectorpointer(Sender: TObject);
    procedure Render(Sender: TObject; var Done : Boolean);
    procedure Reset(Sender: TObject);
    procedure ShowAnzeige(Sender: TObject);
    procedure ShowQualitaet(Sender: TObject);
    procedure ShowQuellen(Sender: TObject);
    procedure StartStop(Sender: TObject);
    procedure StartStop3DMove(Sender: TObject);
    procedure Step(Sender: TObject);
    procedure VecPointerClose(Sender : TObject; var _Action : TCloseAction);
    procedure FourierGraphClose(Sender : TObject; var _Action : TCloseAction);
    procedure D2GraphClose(Sender : TObject; var _Action : TCloseAction);
  private
    { private declarations }
  public
    { public declarations }
  end;
  TFormDX = class(TForm)
   private
    procedure FullscreenDblClick(Sender : TObject);
   protected
    procedure DoClose(var CloseAction : TCloseAction); override;
   public
    constructor Create(_Owner : TControl);
  end;

var
  FormSim : TFormSim;
  Steps   : Cardinal = 0;
  _i      : integer;
  FullLoaded    : Boolean = false;
  Fullscreen    : Boolean = false;
  D3DDev9       : IDirect3DDevice9;
  D3D9          : IDirect3D9;
  d3dpp         : TD3DPresent_Parameters;
  VecPointers   : Array of TVectorPointer;
  FourierGraphs : Array of TFourierGraph;
  D2Graphs      : Array of T2DGraph;
  FormDX        : TFormDX = nil;

  LastTickCount : Cardinal;
  TimeDiff      : Cardinal;

  MatView2D,MatProj2D,MatView3D,MatProj3D : TD3DXMatrix;
  pos, dir, up : TD3DXVector3;

  AccelerationTime : Cardinal = 500;
  MaxSpeed : Single = 0.001;

implementation

{$R *.lfm}

procedure DXRenderSet;
begin
 D3DDev9.SetRenderState(D3DRS_LIGHTING,0);
 d3ddev9.SetRenderState(D3DRS_CULLMODE,D3DCULL_NONE);
 d3ddev9.SetRenderState(D3DRS_Alphablendenable,1);
 d3ddev9.SetRenderState(D3DRS_BLENDFACTOR,D3DColor_XRGB(100,100,100));
 d3ddev9.SetRenderState(D3DRS_ALPHATESTENABLE,1);
 d3ddev9.SetRenderState(D3DRS_ALPHAREF,10);
 d3ddev9.SetRenderState(D3DRS_ALPHAFUNC,D3DCMP_GREATER);
 D3DXMatrixPerspectiveFovLH(MatProj3D,D3DX_PI / 4,FormDX.width / FormDX.height,0.01,20000);
end;




type
 TMove3D = record
  Started  : Boolean;
  Center,P : TPoint;
  Diff     : TD3DXVector2;
  D,O      : TD3DXVector3;
  l,s      : Single;
  KTop     : Integer;
  KLeft    : Integer;
  KRight   : Integer;
  KBottom  : Integer;
  KForward : Integer;
  KBack    : Integer;
  KRot1    : Byte;
  KRot2    : Byte;
 end;
var _Move3D : TMove3D;

procedure TFormSim.Move3D;
begin with (_Move3D) do begin
 if (KTop > 0) then begin KTop := KTop + TimeDiff; if (KTop > AccelerationTime) then KTop := AccelerationTime; end;
 if (KTop < 0) then begin KTop := KTop + TimeDiff; if (KTop > 0)    then KTop := 0; end;
 if (KLeft > 0) then begin KLeft := KLeft + TimeDiff; if (KLeft > AccelerationTime) then KLeft := AccelerationTime; end;
 if (KLeft < 0) then begin KLeft := KLeft + TimeDiff; if (KLeft > 0)    then KLeft := 0; end;
 if (KRight > 0) then begin KRight := KRight + TimeDiff; if (KRight > AccelerationTime) then KRight := AccelerationTime; end;
 if (KRight < 0) then begin KRight := KRight + TimeDiff; if (KRight > 0)    then KRight := 0; end;
 if (KBottom > 0) then begin KBottom := KBottom + TimeDiff; if (KBottom > AccelerationTime) then KBottom := AccelerationTime; end;
 if (KBottom < 0) then begin KBottom := KBottom + TimeDiff; if (KBottom > 0)    then KBottom := 0; end;
 if (KForward > 0) then begin KForward := KForward + TimeDiff; if (KForward > AccelerationTime) then KForward := AccelerationTime; end;
 if (KForward < 0) then begin KForward := KForward + TimeDiff; if (KForward > 0)    then KForward := 0; end;
 if (KBack > 0) then begin KBack := KBack + TimeDiff; if (KBack > AccelerationTime) then KBack := AccelerationTime; end;
 if (KBack < 0) then begin KBack := KBack + TimeDiff; if (KBack > 0)    then KBack := 0; end;


 GetCursorPos(P);
 Center.X := 0;
 Center.Y := 0;
 Center := FormDX.ClientToScreen(Center);
 Center.X := Center.X + FormDX.Width div 2;
 Center.Y := Center.Y + FormDX.Height div 2;
 Diff.X := (Center.X - P.X) / Screen.Height;
 Diff.Y := (Center.Y - P.Y) / Screen.Height;
 SetCursorPos(Center.X,Center.Y);
 O.X := D.Y * Up.Z - D.Z * Up.Y;
 O.Y := -(D.X * Up.Z - D.Z * Up.X);
 O.Z := D.X * Up.Y - D.Y * Up.X;
 D.X := D.X + O.X * Diff.X + Up.X * Diff.Y;
 D.Y := D.Y + O.Y * Diff.X + Up.Y * Diff.Y;
 D.Z := D.Z + O.Z * Diff.X + Up.Z * Diff.Y;
 s := sqrt(D.X*D.X + D.Y*D.Y + D.Z*D.Z);
 D.X := D.X / s;
 D.Y := D.Y / s;
 D.Z := D.Z / s;
 Up.X := -(D.Y * O.Z - D.Z * O.Y)  +  O.X * (KRot1 - KRot2) * 0.001 * TimeDiff;
 Up.Y :=   D.X * O.Z - D.Z * O.X   +  O.Y * (KRot1 - KRot2) * 0.001 * TimeDiff;
 Up.Z := -(D.X * O.Y - D.Y * O.X)  +  O.Z * (KRot1 - KRot2) * 0.001 * TimeDiff;
 s := sqrt(Up.X*Up.X + Up.Y*Up.Y + Up.Z*Up.Z);
 Up.X := Up.X / s;
 Up.Y := Up.Y / s;
 Up.Z := Up.Z / s;
 Pos.X := Pos.X + (D.X * (abs(KForward) - abs(KBack)) + Up.X * (abs(KTop) - abs(KBottom)) + O.X * (abs(KLeft) - abs(KRight))) / AccelerationTime * MaxSpeed * TimeDiff;
 Pos.Y := Pos.Y + (D.Y * (abs(KForward) - abs(KBack)) + Up.Y * (abs(KTop) - abs(KBottom)) + O.Y * (abs(KLeft) - abs(KRight))) / AccelerationTime * MaxSpeed * TimeDiff;
 Pos.Z := Pos.Z + (D.Z * (abs(KForward) - abs(KBack)) + Up.Z * (abs(KTop) - abs(KBottom)) + O.Z * (abs(KLeft) - abs(KRight))) / AccelerationTime * MaxSpeed * TimeDiff;

 Dir.X := Pos.X + l*D.X;
 Dir.Y := Pos.Y + l*D.Y;
 Dir.Z := Pos.Z + l*D.Z;
 D3DXMatrixLookAtLH(MatView3D,pos,dir,up);
 D3DXMatrixPerspectiveFovLH(MatProj3D,D3DX_PI / 4,FormDX.width / FormDX.height,0.01,20000);
 EditMove3D.SetFocus;
end; end;

constructor TFormDX.Create(_Owner : TControl);
begin
 inherited Create(TComponent(_Owner));
 BorderStyle := bsNone;
 OnClick := @FormSim.StartStop3DMove;
 OnKeyDown := @FormSim.Move3DKeyDown;
 OnKeyUp := @FormSim.Move3DKeyUp;
 OnDblClick := @FullscreenDblClick;
end;

procedure TFormDX.FullscreenDblClick(Sender : TObject);
var w : Word;
begin
 w := ContSet.KFullscreen;
 FormSim.Move3DKeyDown(Sender,w,[]);
end;

procedure TFormDX.DoClose(var CloseAction : TCloseAction);
begin
 CloseAction := caFree;
 FormDX := nil;
end;

procedure TFormSim.Move3DKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin with (_Move3D) do begin
 if (not Started) then exit;
 if (Key = ContSet.KFullscreen) then begin
  if (Fullscreen) then begin
   Fullscreen := false;
   d3dpp.Windowed := true;
   d3dpp.BackBufferWidth := FormDX.Width;
   d3dpp.BackBufferHeight := FormDX.Height;
   d3ddev9.Reset(d3dpp);
   DXRenderSet;
   FormDX.WindowState := wsNormal;
   FormDX.FormStyle := fsNormal;
   Windows.SetParent(FormDX.Handle,Panel1.Handle);
   Screen.Cursor := crDefault;
  end else begin
   Fullscreen := true;
   Windows.SetParent(FormDX.Handle,0);
   FormDX.WindowState := wsMaximized;
   FormDX.FormStyle := fsStayOnTop;
   d3dpp.Windowed := false;
   d3dpp.BackBufferWidth := Screen.Width;
   d3dpp.BackBufferHeight := Screen.Height;
   d3ddev9.Reset(d3dpp);
   DXRenderSet;
   Screen.Cursor := crNone;
  end;
 end;
 if (Key = ContSet.KLeft) then if (KLeft <= 0) then KLeft := abs(KLeft) + 1;
 if (Key = ContSet.KRight) then if (KRight <= 0) then KRight := abs(KRight) + 1;
 if (Key = ContSet.KForward) then if (KForward <= 0) then KForward := abs(KForward) + 1;
 if (Key = ContSet.KBack) then if (KBack <= 0) then KBack := abs(KBack) + 1;
 if (Key = ContSet.KTop) then if (KTop <= 0) then KTop := abs(KTop) + 1;
 if (Key = ContSet.KBottom) then if (KBottom <= 0) then KBottom := abs(KBottom) + 1;
 if (Key = ContSet.KRot1) then KRot1 := 1;
 if (Key = ContSet.KRot2) then KRot2 := 1;
 if (Key = ContSet.KCenter) then begin
  Dir.X := 0;
  Dir.Y := 0;
  Dir.Z := 0;
  D.X := Dir.X - Pos.X;
  D.Y := Dir.Y - Pos.Y;
  D.Z := Dir.Z - Pos.Z;
 end;
end; end;

procedure TFormSim.Move3DKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin with (_Move3D) do begin
 if (Key = ContSet.KLeft) then KLeft := - abs(KLeft);
 if (Key = ContSet.KRight) then KRight := - abs(KRight);
 if (Key = ContSet.KForward) then KForward := - abs(KForward);
 if (Key = ContSet.KBack) then KBack := - abs(KBack);
 if (Key = ContSet.KTop) then KTop := - abs(KTop);
 if (Key = ContSet.KBottom) then KBottom := - abs(KBottom);
 if (Key = ContSet.KRot1) then KRot1 := 0;
 if (Key = ContSet.KRot2) then KRot2 := 0;
end; end;

procedure TFormSim.StartStop3DMove(Sender: TObject);
begin with (_Move3D) do begin
 if (Started) then begin
  if (Fullscreen) then exit;
  Started := false;
  FormDX.Cursor := crDefault;
 end else begin
  if (not CheckBox1.Checked) then exit;
  Started := true;
  KLeft := 0;
  KRight := 0;
  KTop := 0;
  KBottom := 0;
  KRot1 := 0;
  KRot2 := 0;
  FormDX.Cursor := crNone;
  D.X := Dir.X - Pos.X;
  D.Y := Dir.Y - Pos.Y;
  D.Z := Dir.Z - Pos.Z;
  l := sqrt(D.X*D.X + D.Y*D.Y + D.Z*D.Z);
  D.X := D.X / l;
  D.Y := D.Y / l;
  D.Z := D.Z / l;
 end;
end; end;

// Funktion ist von http://www.swissdelphicenter.ch/de/showcode.php?id=152
function CopyDir(const fromDir, toDir: string): Boolean;
var
   fos: TSHFileOpStruct;
begin
   ZeroMemory(@fos, SizeOf(fos));
   with fos do
   begin
     wFunc  := FO_COPY;
     fFlags := FOF_FILESONLY;
     pFrom  := PChar(fromDir + #0);
     pTo    := PChar(toDir)
   end;
   Result := (0 = ShFileOperation(fos));
end;


procedure TFormSim.FormCreate(Sender: TObject);
var i : integer;
begin
 Panel1.Caption := 'Lädt...';

 DefaultSettings;
 SetDir := ExtractFilePath(ParamStr(0));
 if (FileExists(SetDir + 'Settings\Settings.ini')) then SetDir := SetDir  + 'Settings\'
 else begin
  SetLength(SetDir,MAX_PATH);
  ExpandEnvironmentStrings('%appdata%',PChar(SetDir),Length(SetDir) + 2);
  for i := 1 to Length(SetDir) do if (Byte(SetDir[i]) = 0) then break;
  SetLength(SetDir,i-1);
  if (SetDir[i] <> '\') then SetDir := SetDir + '\';
  SetDir := SetDir + 'Welleninterferenz\';
 end;
 if (not FileExists(SetDir + 'Settings.ini')) then begin
  if (MessageDlg(
    'Die Anwendungsdaten wurden nicht gefunden. Entweder wurde das Programm zum ersten Mal von diesem Benutzer gestartet oder nicht richtig installiert. Möchten Sie die Anwendungsdaten in diesem Benutzerkonto anlegen?',
    MtCustom, [MbYes, MbNo], 0) = mrYes) then begin
     if (not DirectoryExists(PChar(ExtractFilePath(ParamStr(0)) + 'StdSettings\'))) then ShowMessage('Fehler: Die Standardeinstallungen wurden nicht gefunden. Neuinstallation der Anwendung kann das Problem beheben.')
     else CopyDir(ExtractFilePath(ParamStr(0)) + 'StdSettings\',SetDir);
    end else SetDir := '';
 end else LoadSettings;

 _Move3D.Started := false;
 FormDX := TFormDX.Create(self);
 FormDX.Width := Panel1.Width;
 FormDX.Height := Panel1.Height;
 Windows.SetParent(FormDX.Handle,Panel1.Handle);
 Timer1.Enabled := true;
end;

procedure TFormSim.Change3DSet(Sender: TObject);
var b : Boolean;
begin
 if (FormDX.Cursor = crCross) then CheckBox1.Checked := false;
 Render(Sender,b);
 Is3DOn := CheckBox1.Checked;
 Button3.Enabled := not CheckBox1.Checked;
 Button4.Enabled := not CheckBox1.Checked;
 Button8.Enabled := not CheckBox1.Checked;
end;

procedure TFormSim.Load(Sender: TObject);
var
  dtype            : TD3DDevType;
  D3DCaps9         : TD3DCaps9;
  vertexprocessing : Integer;
  d3ddm            : TD3DDisplayMode;
  b : Boolean;
  i : integer;
  p : PBuffer;
  e : Cardinal;

begin
 if (not Timer1.Enabled) then exit;
 Timer1.Enabled := false;

 // Create DirectX
 d3d9 := Direct3DCreate9(D3D_SDK_VERSION);
 if (d3d9 = nil) then  begin Panel1.Caption := 'Fehler beim erstellen von Direct3D.'; exit; end;
 Fillchar(d3dpp,sizeof(d3dpp),0);
 with (d3dpp) do begin
  SwapEffect := D3DSWAPEFFECT_DISCARD;
  Windowed := true;
  e := D3D9.GetAdapterDisplayMode(D3DADAPTER_DEFAULT,d3ddm);
  if (failed(e)) then begin Panel1.Caption := 'Fehler beim erstellen abfragen das Display Modes. Fehlercode 0x' + IntToHex(e,8); exit; end;
  BackBufferFormat := d3ddm.Format;
  BackBufferWidth  := FormDX.width;
  BackBufferHeight := FormDX.height;
  EnableAutoDepthStencil := true;
  AutoDepthStencilFormat := D3DFMT_D16;
 end;
 i := 0;
 repeat begin
  case (i) of
   0 : dtype := D3DDEVTYPE_HAL;
   1 : dtype := D3DDEVTYPE_REF;
   2 : dtype := D3DDEVTYPE_NULLREF;
  end;
  e := d3d9.GetDeviceCaps(D3DADAPTER_DEFAULT,dtype,D3DCaps9);
  if (failed(e)) then begin Panel1.Caption := 'Fehler beim erstellen abfragen von Direct3D Device Caps. Fehlercode 0x' + IntToHex(e,8); exit; end;
  b := D3DCaps9.DevCaps and D3DDEVCAPS_HWTRANSFORMANDLIGHT <> 0;
  if (b) then vertexprocessing := D3DCREATE_HARDWARE_VERTEXPROCESSING
  else vertexprocessing := D3DCREATE_SOFTWARE_VERTEXPROCESSING;
  e := d3d9.CreateDevice(D3DADAPTER_DEFAULT,dtype,FormDX.Handle,vertexprocessing,@d3dpp,d3ddev9);
  inc(i);
 end until (not failed(e)) or (i > 2);
 if (failed(e)) then begin Panel1.Caption := 'Fehler beim erstellen von Direct3DDevice. Fehlercode 0x' + IntToHex(e,8); exit; end;

 // Simulator erstellen
 Sim := TWaveSim.Create(400,300,D3DDev9);
 Sim.onStop := @onStop;
 Sim.onStepReady := @onStepReady;

 // SetCamera
 pos := D3DXVector3(0,0,-10000);
 dir := D3DXVector3(0.0,0.0,0.0);
 up  := D3DXVector3(0.0,1.0,0.0);
 D3DXMatrixLookAtLH(MatView2D,pos,dir,up);
 D3DXMatrixOrthoLH(MatProj2D,2,2,0.01,20000);
 pos := D3DXVector3(-1,-2,-2);
 dir := D3DXVector3(0.0,0.0,0.0);
 up  := D3DXVector3(0.0,1.0,0.0);
 D3DXMatrixLookAtLH(MatView3D,pos,dir,up);

 // Set DX-Parameters
 DXRenderSet;

 // Final Settings
 LastTickCount := GetTickCount;
 Application.OnIdle := @Render;
 Panel1.Caption := '';
 FormDX.Show;
 Button1.Enabled := true;
 Button2.Enabled := true;
 Button9.Enabled := true;
 Button5.Enabled := true;
 Button6.Enabled := true;
 Button7.Enabled := true;
 Button10.Enabled := true;
 FullLoaded := true;
end;

procedure TFormSim.PanelResize(Sender: TObject);
var b : Boolean;
begin
 if (not FullLoaded) or (Fullscreen) then exit;
 FormDX.Top := 0;
 FormDX.Left := 0;
 FormDX.Width := Panel1.Width;
 FormDX.Height := Panel1.Height;
 d3dpp.BackBufferWidth  := FormDX.width;
 d3dpp.BackBufferHeight := FormDX.height;
 d3dpp.Windowed := true;
 d3ddev9.Reset(d3dpp);
 DXRenderSet;
 Render(Sender,b);
end;

procedure TFormSim.ButSteuerungClick(Sender: TObject);
begin
 if (FormControls <> nil) then begin FormControls.Show; FormControls.BringToFront; end
 else FormControls := TFormControls.Create(self);
end;

procedure TFormSim.PlaceFourierGraph(Sender: TObject);
var i : integer;
begin
 i := Length(FourierGraphs);
 SetLength(FourierGraphs,i+1);
 FourierGraphs[i] := TFourierGraph.Create(Sim,FormDX,D3Ddev9);
 FourierGraphs[i].OnClose := @FourierGraphClose;
end;

procedure TFormSim.PlaceGraph2D(Sender: TObject);
var i : integer;
begin
 i := Length(D2Graphs);
 SetLength(D2Graphs,i+1);
 D2Graphs[i] := T2DGraph.Create(Sim,FormDX,D3Ddev9);
 D2Graphs[i].OnClose := @D2GraphClose;
end;

procedure TFormSim.PlaceVectorpointer(Sender: TObject);
var i : integer;
begin
 i := Length(VecPointers);
 SetLength(VecPointers,i+1);
 VecPointers[i] := TVectorPointer.Create(Sim,FormDX,D3Ddev9);
 VecPointers[i].OnClose := @VecPointerClose;
end;

procedure TFormSim.VecPointerClose(Sender : TObject; var _Action : TCloseAction);
var
 i : integer;
 b : boolean;
begin
 b := false;
 for i := 0 to Length(VecPointers)-1 do begin
  if (VecPointers[i] = Sender) then b := true;
  if (b) and (i < Length(VecPointers)-1) then VecPointers[i] := VecPointers[i+1];
 end;
 if (b) then SetLength(VecPointers,i);
end;

procedure TFormSim.FourierGraphClose(Sender : TObject; var _Action : TCloseAction);
var
 i : integer;
 b : boolean;
begin
 b := false;
 for i := 0 to Length(FourierGraphs)-1 do begin
  if (FourierGraphs[i] = Sender) then b := true;
  if (b) and (i < Length(FourierGraphs)-1) then FourierGraphs[i] := FourierGraphs[i+1];
 end;
 if (b) then SetLength(FourierGraphs,i);
end;

procedure TFormSim.D2GraphClose(Sender : TObject; var _Action : TCloseAction);
var
 i : integer;
 b : boolean;
begin
 b := false;
 for i := 0 to Length(D2Graphs)-1 do begin
  if (D2Graphs[i] = Sender) then b := true;
  if (b) and (i < Length(D2Graphs)-1) then D2Graphs[i] := D2Graphs[i+1];
 end;
 if (b) then SetLength(D2Graphs,i);
end;

var ROverlapping : ShortInt;
    BGColor      : Cardinal;

procedure TFormSim.Render(Sender: TObject; var Done : Boolean);
begin
 TimeDiff := GetTickCount - LastTickCount;
 LastTickCount := GetTickCount;
 if (_Move3D.Started) then Move3D;
 if (CheckBox1.Checked) then begin
  d3ddev9.SetTransform(D3DTS_VIEW,MatView3D);
  d3ddev9.SetTransform(D3DTS_PROJECTION,MatProj3D);
  ROverlapping := 2;
 end else begin
  d3ddev9.SetTransform(D3DTS_VIEW,MatView2D);
  d3ddev9.SetTransform(D3DTS_PROJECTION,MatProj2D);
  ROverlapping := 0;
 end;
 D3DDev9.BeginScene;
 PByte(@BGColor+0)^ := PByte(@_Settings.BGColor + 2)^;
 PByte(@BGColor+1)^ := PByte(@_Settings.BGColor + 1)^;
 PByte(@BGColor+2)^ := PByte(@_Settings.BGColor + 0)^;
 d3ddev9.Clear(0,nil,D3DCLEAR_TARGET or D3DCLEAR_ZBUFFER,BGColor,1.0,0);

 if (Sim.CurrentBuf <> nil) and (Sim.CurrentInd <> nil) then begin
  d3ddev9.SetRenderState(D3DRS_SRCBLEND,D3DBLEND_SRCALPHA);
  d3ddev9.SetRenderState(D3DRS_DESTBLEND,D3DBLEND_INVSRCALPHA);
  d3ddev9.SetStreamSource(0,Sim.CurrentBuf,0,sizeof(TVertex));
  d3ddev9.SetFVF(D3DFVF_XYZ or D3DFVF_DIFFUSE);
  d3ddev9.SetIndices(Sim.CurrentInd);
  d3ddev9.DrawIndexedPrimitive(d3dpt_trianglelist,0,0,Sim.Width * Sim.Height,0,(Sim.Width - 1) * (Sim.Height - 1) * 2);
 end;

 repeat begin
  if (ROverlapping < 2) then d3ddev9.Clear(0,nil,D3DCLEAR_ZBUFFER,$000000,1.0,0);
  if (ROverlapping = 1) then begin
   d3ddev9.SetRenderState(D3DRS_SRCBLEND,D3DBLEND_BLENDFACTOR);
   d3ddev9.SetRenderState(D3DRS_DESTBLEND,D3DBLEND_INVBLENDFACTOR);
  end else begin
   d3ddev9.SetRenderState(D3DRS_SRCBLEND,D3DBLEND_ONE);
   d3ddev9.SetRenderState(D3DRS_DESTBLEND,D3DBLEND_ZERO);
  end;

  _i := 0;
  while (_i < Length(VecPointers)) do begin
   VecPointers[_i].Render;
   inc(_i);
  end;

  _i := 0;
  while (_i < Length(FourierGraphs)) do begin
   FourierGraphs[_i].Render;
   inc(_i);
  end;

  _i := 0;
  while (_i < Length(D2Graphs)) do begin
   D2Graphs[_i].Render;
   inc(_i);
  end;

  if (ROverlapping = 1) then begin
   d3ddev9.SetRenderState(D3DRS_SRCBLEND,D3DBLEND_ONE);
   d3ddev9.SetRenderState(D3DRS_DESTBLEND,D3DBLEND_ZERO);
  end;
  dec(ROverlapping);
 end until (ROverlapping < 1);

 D3DDev9.EndScene;
 D3DDev9.Present(nil, nil, FormDX.Handle, nil);
 Done := not _Move3D.Started;
end;

procedure TFormSim.Reset(Sender: TObject);
begin
 Sim.Reset;
 Steps := 0;
end;

procedure TFormSim.ShowAnzeige(Sender: TObject);
begin
 FormDisplay.Visible := true;
 FormDisplay.BringToFront;
end;

procedure TFormSim.ShowQualitaet(Sender: TObject);
begin
 FormQuality.Visible := true;
 FormQuality.BringToFront;
end;

procedure TFormSim.ShowQuellen(Sender: TObject);
begin
 FormSources.Visible := true;
 FormSources.BringToFront;
end;

procedure TFormSim.onStop;
begin
 Button1.Caption := 'Start';
 Button1.Enabled := true;
 Button2.Enabled := true;
 Button9.Enabled := true;
end;

procedure TFormSim.onStepReady;
var b : Boolean;
begin
 inc(Steps);
 Label1.Caption := IntToStr(Steps);
 Render(self,b);
 Application.ProcessMessages;
end;

procedure TFormSim.StartStop(Sender: TObject);
begin
 if (Sim.IsWorking) then begin
  Button1.Enabled := false;
  Sim.BreakCalc;
 end else begin
  Button1.Caption := 'Stop';
  Sim.StartCalc;
  Button2.Enabled := false;
  Button9.Enabled := false;
 end;
end;

procedure TFormSim.Step(Sender: TObject);
begin
 Button2.Enabled := false;
 Button9.Enabled := false;
 Sim.StartCalc;
 Sim.BreakCalc;
end;

procedure TFormSim.FormClose(Sender: TObject; var CloseAction: TCloseAction);
var i : integer;
begin
 if (not FullLoaded) then exit;
 Application.OnIdle := nil;
 if (FormDX <> nil) then FormDX.Free;
 for i := 0 to Length(VecPointers)-1 do begin VecPointers[i].OnClose := nil; VecPointers[i].Free; end;
 for i := 0 to Length(FourierGraphs)-1 do begin FourierGraphs[i].OnClose := nil; FourierGraphs[i].Free; end;
 for i := 0 to Length(D2Graphs)-1 do begin D2Graphs[i].OnClose := nil; D2Graphs[i].Free; end;
 Sim.Free;
end;

end.

