unit WaveSimulation;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, direct3d9, d3dx9, Windows, SyncObjs, Math, Forms, dialogs,
  FarbScala, Settings;


type
 TProcOfObj = procedure of object;
 TWaveSim = class;
 TVertex = packed record
  x,y,z   : Single;
  diffuse : TD3DColor;
 end;
 TBuffer = array [0..1] of TVertex;
 PBuffer = ^TBuffer;

 TWaveSim_Vector = packed record
  Amp : Word;
  Phi : Word;
  Dir : Word;
  Long: Byte;
  Used: Boolean;
 end;
 PWaveSim_Vector = ^TWaveSim_Vector;
 TWaveSim_Vectors = array of TWaveSim_Vector;
 PWaveSim_Vectors = ^TWaveSim_Vectors;

 TWaveSim_Field = record
  Reflect : single;
  Normale : word;
  Absorb  : single;
  Slow    : single;
  Source  : Byte;
  Height  : integer;
  V : Array [0..1] of TWaveSim_Vectors;
 end;
 PWaveSim_Field = ^TWaveSim_Field;

 TWaveSim_Calc = class(TThread)
  private
   Parent   : TWaveSim;
   X,Y      : word;
  protected
   TWorking : Boolean;
   procedure Execute; override;
  public
   constructor Create(_Parent : TWaveSim);
   procedure WorkRow(Row : Word);
 end;

 TFourierPart = record
  Amp  : Word;
  Long : byte;
  Phi0 : word;
  Phi  : word;
 end;

 TWaveSim_Source = record
  Art     : Byte;
  Dir     : Word;
  Fourier : Array of TFourierPart;
 end;
 PWaveSim_Source = ^TWaveSim_Source;

 TWaveSim = class
  private
   D3DDev9      : IDirect3DDevice9;
   Buf          : Array [0..1] of IDirect3DVertexbuffer9;
   Ind          : Array [0..1] of IDirect3DIndexbuffer9;
   LockRect     : TRect;
   LockRectB    : TD3DLOCKED_RECT;
   Stop         : Boolean;
   DoClose      : Boolean;
   Working      : Boolean;
   destructor Destroy; override;
  protected
   Count         : Cardinal;
   Offset        : Byte;
   N,O           : Byte;
   Dat           : Array of Array of TWaveSim_Field;
   Threads       : Array of TWaveSim_Calc;
   Row           : integer;
   Switched      : Boolean;
   SecGetNewWork : TRTLCriticalSection;
   UnusedVector  : TWaveSim_Vector;
   CPA           : Array of Word;
   MinAmp        : integer;
   procedure GetNewWork(Sender : TWaveSim_Calc);
   procedure Switch;
   procedure EndCalc;
  public
   Sources      : Array of TWaveSim_Source;
   CurrentBuf   : IDirect3DVertexbuffer9;
   CurrentInd   : IDirect3DIndexbuffer9;
   ShownWaveAmp : Cardinal;
   onStop       : TProcOfObj;
   onStepReady  : TProcOfObj;
   onMapLoad    : TProcOfObj;
   Width,Height : Word;
   ReduceCalculationCapacity : Boolean;
   WavelikeSpread            : Byte;
   ShowBlur                  : Boolean;
   constructor Create(_Width,_Height : Word; Device : IDirect3DDevice9);
   procedure Reset;
   procedure LoadToTex;
   procedure StartCalc;
   procedure CopyVectors(pX,pY : integer; var Vec : TWaveSim_Vectors);
   property IsWorking : Boolean read Working;
   procedure BreakCalc;
   procedure Free;
 end;

var
 Sim : TWaveSim;

const
 SrcLaser = 1;
 SrcCenter = 2;

implementation

const
 LengthOfVecBuf = 10;
 SquareResolution = 1000;
 TrigRes = 1000;
 DirImprecision = 100;
 CircleSourceN = 100;

type
 T3Vertex = Array [0..2] of Cardinal;
 TIBuffer = Array [0..1] of T3Vertex;
 PIBuffer = ^TIBuffer;

constructor TWaveSim.Create(_Width,_Height : Word; Device : IDirect3DDevice9);
var
 SysInfo : TSystemInfo;
 i       : integer;


procedure CreateSquare;
var
 i,j,p       : integer;
 d,h,r,s1,s2 : Single; // x = P1.x    y = P2.y    r = r    s = x1
function F(x : single) : single;
begin
 Result := r*r*arcsin(x/r)/2 + x*sqrt(r*r-x*x)/2 + x*h;
end;
function G(x : single) : single;
begin
 Result := -r*r*arcsin(x/r)/2 - x*sqrt(r*r-x*x)/2 + x*h;
end;
begin
 r := 0.56418958354775628694807945156077;
 SetLength(CPA,SquareResolution * SquareResolution);
 for i := 0 to SquareResolution - 1 do for j := 0 to SquareResolution - 1 do begin
  p := j * SquareResolution + i;
  d := r - (i / (SquareResolution-1)) * 2*r;
  h := (0.5 - (j / (SquareResolution-1))) * 2*r;
  s1 := - sqrt(r*r - h*h);
  s2 :=  sqrt(r*r - h*h);
  if (h >= 0) then begin
   if (d > s1) and (d < s2) then CPA[p] := Round( -(G(d) - G(s1)) * 65535)
   else if (d > s1) and (d >= s2) then CPA[p] := Round( -(G(s2) - G(s1)) * 65535)
   else CPA[p] := 0;
  end else begin
   CPA[p] := Round( -(G(d) - G(-r)) * 65535);
   if (s1 < d) then CPA[p] := CPA[p] - Round( -(F(s1) - F(-r)) * 65535);
   if (s2 < d) then CPA[p] := CPA[p] - Round( -(F(d) - F(s2)) * 65535);
   if (d <= s1) then CPA[p] := CPA[p] - Round( -(F(d) - F(-r)) * 65535);
  end;
 end;
end;

procedure InitGeometry;
var
  p : PBuffer;
  q : PIBuffer;
  x, y : Cardinal;
  c : Cardinal;
begin
 // Erstellten der Vertices
 d3ddev9.CreateVertexBuffer(Width * Height * sizeof(TVertex),0,D3DFVF_XYZ or D3DFVF_DIFFUSE,D3DPool_MANAGED,Buf[N],nil);
 Buf[N].Lock(0,Width * Height * sizeof(TVertex),p,0);
 for x := 0 to Width - 1 do for y := 0 to Height - 1 do begin
  p^[y * Width + x].x := x / (Width-1) * 2 - 1;
  p^[y * Width + x].y := - y / (Height-1) * 2 + 1;
  p^[y * Width + x].z := 0;
  p^[y * Width + x].diffuse := $FFFF00FF;
 end;
 Buf[N].UnLock;

 // Erstellen der Indizes
 d3ddev9.CreateIndexBuffer((Width-1) * (Height-1) * 2 * SizeOf(T3Vertex),0,D3DFMT_INDEX32,D3DPOOL_MANAGED,Ind[N],nil);
 Ind[N].Lock(0,(Width-1) * (Height-1) * 2 * SizeOf(T3Vertex),q,0);
 c := 0;
 for y := 0 to Height - 2 do for x := 0 to Width - 2 do begin
  q^[c][0] := y * Width + x;
  q^[c][1] := y * Width + x+1;
  q^[c][2] := (y+1) * Width + x;
  inc(c);
  q^[c][0] := y * Width + x+1;
  q^[c][1] := (y+1) * Width + x;
  q^[c][2] := (y+1) * Width + x+1;
  inc(c);
 end;
 Ind[N].UnLock;
end;

begin
 inherited Create;
 CurrentBuf := nil;
 CurrentInd := nil;
 D3DDev9 := Device;
 if (_Width mod 2 <> 0) then _Width := _Width + 1;
 if (_Height mod 2 <> 0) then _Height := _Height + 1;
 Width := _Width;
 Height := _Height;
 ShownWaveAmp := 65536;
 Stop := false;
 onStop := nil;
 onStepReady := nil;
 onMapLoad := nil;
 DoClose := false;
 Count := 0;
 MinAmp := 100;
 Offset := 0;
 ReduceCalculationCapacity := false;
 WavelikeSpread := 4;
 ShowBlur := true;

 N := 0; InitGeometry;
 N := 1; InitGeometry;

 SetLength(Dat,Width+1);
 for i := 0 to Width do begin
  SetLength(Dat[i],Height+1);
  ZeroMemory(@Dat[i][0],SizeOf(TWaveSim_Field) * (Height+1));
 end;
 N := 0;
 O := 1;

 CreateSquare;
 InitializeCriticalSection(SecGetNewWork);
 GetSystemInfo(SysInfo);
 SetLength(Threads,SysInfo.dwNumberOfProcessors);                          // SysInfo.dwNumberOfProcessors
 for i := 0 to Length(Threads) - 1 do Threads[i] := TWaveSim_Calc.Create(self);

 Reset;
end;

procedure TWaveSim.Reset;
var
 x,y,i,j : integer;
 Map     : Byte;
begin
 if (Working) then exit;
 Count := 0;
 for x := 0 to width do begin
  for y := 0 to Height do begin
   SetLength(Dat[x][y].V[0],0);
   SetLength(Dat[x][y].V[1],0);
  end;
  ZeroMemory(@Dat[x][0],SizeOf(TWaveSim_Field) * (Height+1));
 end;

 try
  Map := StrToInt(InputBox('Nummer der zu simulierenden Karte','Bitte geben Sie die Nummer der zu simulierenden Karte ein','0'));
 except
  if (onMapLoad <> nil) then onMapLoad;
  exit;
 end;

 // LoadMap
 case (Map) of
  0 : begin
   SetLength(Sources,2);
   Sources[0].Art := SrcCenter;
   SetLength(Sources[0].Fourier,1);
   Sources[0].Fourier[0].Amp := 60000;
   Sources[0].Fourier[0].Long := 20;
   Sources[0].Fourier[0].Phi0 := 0;
   Sources[1].Art := SrcCenter;
   SetLength(Sources[1].Fourier,1);
   Sources[1].Fourier[0].Amp := 60000;
   Sources[1].Fourier[0].Long := 20;
   Sources[1].Fourier[0].Phi0 := 0;
   Dat[Width div 3][Height div 2].Source := 1;
   Dat[Width div 3 * 2+1][Height div 2].Source := 2;
   for i := Width div 4 * 3 to Width - 1 do for j := Height div 4 * 3 to Height - 1 do Dat[i][j].Slow := 0.5;
   for i := 0 to Width div 4 - 1 do begin Dat[i + Width div 4 * 3][Height div 4 + i div 2].Reflect := 1; Dat[i + Width div 4 * 3][Height div 4 + i div 2].Normale := 10655; end;
   for i := 0 to Width div 4 - 1 do begin Dat[i + Width div 4 * 3][Height div 4 + i div 2 + 1].Reflect := 1; Dat[i + Width div 4 * 3][Height div 4 + i div 2 + 1].Normale := 10655; end;
   for i := 0 to Width div 4 - 2 do begin Dat[i + Width div 4 * 3 + 1][Height div 4 + i div 2 + 1].Reflect := 1; Dat[i + Width div 4 * 3][Height div 4 + i div 2 + 1].Normale := 10655; end;
   for i := Height div 5 * 2 to Height div 5 * 3 do begin Dat[Width div 6][i].Absorb := 1; Dat[Width div 6+1][i].Absorb := 1; end;
  end;
  1 : begin
   SetLength(Sources,1);
   Sources[0].Art := SrcLaser;
   Sources[0].Dir := 57344;
   SetLength(Sources[0].Fourier,1);
   Sources[0].Fourier[0].Amp := 60000;
   Sources[0].Fourier[0].Long := 100;
   Sources[0].Fourier[0].Phi0 := 0;
   for i := 0 to 40 do begin Dat[Width div 8 + i][Height div 3 - i].Source := 1; Dat[Width div 8 + i - 1][Height div 3 - i].Source := 1; end;
   for i := 0 to Width - 1 do for j := Height div 3 * 2 to Height - 1 do Dat[i][j].Slow := 0.3;
  end;
  2 : begin
   SetLength(Sources,1);
   Sources[0].Art := SrcLaser;
   Sources[0].Dir := 8192;
   SetLength(Sources[0].Fourier,1);
   Sources[0].Fourier[0].Amp := 60000;
   Sources[0].Fourier[0].Long := 100;
   Sources[0].Fourier[0].Phi0 := 0;
   for i := 0 to 40 do begin Dat[Width div 8 + i][Height div 3 * 2 + i].Source := 1; Dat[Width div 8 + i + 1][Height div 3 * 2 + i].Source := 1; end;
   for i := 0 to Width - 1 do for j := Height div 2 to Height - 1 do Dat[i][j].Slow := 0.3;
  end;
  3 : begin
   SetLength(Sources,1);
   Sources[0].Art := SrcLaser;
   Sources[0].Dir := 4836;
   SetLength(Sources[0].Fourier,1);
   Sources[0].Fourier[0].Amp := 60000;
   Sources[0].Fourier[0].Long := 100;
   Sources[0].Fourier[0].Phi0 := 0;
   for i := 0 to 40 do begin Dat[Width div 8 + i div 2][Round(Height * 0.6) + i].Source := 1; Dat[Width div 8 + i div 2 + 1][Round(Height * 0.6) + i].Source := 1; end;
   for i := 0 to Width - 1 do for j := Height div 2 to Height - 1 do Dat[i][j].Slow := 0.4;
  end;
 end;
 if (onMapLoad <> nil) then onMapLoad;
end;

procedure TWaveSim.StartCalc;
var
 i,j : integer;
 c   : uint64;
begin
 if (Working) then exit;
 if (Stop) or (DoClose) then begin
  if (onStop <> nil) then onStop;
  Working := false;
  Stop := false;
  exit;
 end;
 Working := true;
 Switched := false;
 Offset := Trunc(Random * 4);
 for i := 0 to Length(Sources) - 1 do for j := 0 to Length(Sources[i].Fourier)-1 do begin
  Sources[i].Fourier[j].Phi := Round((Count mod Sources[i].Fourier[j].Long) / Sources[i].Fourier[j].Long * 65535 + Sources[i].Fourier[j].Phi0) mod 65536;
 end;
 Row := 0;
 Switched := false;
 if (Width = 0) then EndCalc
 else for i := 0 to Length(Threads) - 1 do begin
  Threads[i].WorkRow(Row);
  Threads[i].Resume;
  inc(Row,4);
  if (Row >= Width - 1) then exit;
 end;
end;

procedure TWaveSim.GetNewWork(Sender : TWaveSim_Calc);
begin
 if (Row >= Width - 1) then exit;
 Sender.WorkRow(Row);
 inc(Row,4);
end;

procedure TWaveSim.Switch;
var i : integer;
begin
 Row := 2;
 Switched := true;
 for i := 0 to Length(Threads) - 1 do begin
  Threads[i].WorkRow(Row);
  Threads[i].Resume;
  inc(Row,4);
  if (Row >= Width - 1) then exit;
 end;
end;

procedure TWaveSim.EndCalc;
begin
 LoadToTex;
 inc(Count);
 O := N;
 if (N = 0) then N := 1 else N := 0;
 if (O = N) then begin if (N = 0) then N := 1 else N := 0; end;
 if (onStepReady <> nil) then onStepReady;
 Working := false;
 Application.ProcessMessages;
 if (DoClose) then Free
 else StartCalc;
end;

procedure TWaveSim.BreakCalc;
begin
 Stop := true;
end;

procedure TWaveSim.CopyVectors(pX,pY : integer; var Vec : TWaveSim_Vectors);
var i,j : integer;
begin
 if (PX < 0) or (PX >= Width) or (PY < 0) or (PY >= Height) then exit;
 for j := Length(Dat[PX][pY].V[O]) - 1 downto 0 do if (Dat[PX][PY].V[O][j].Long <> 0) then break;
 SetLength(Vec,j+1);
 for i := 0 to j do Vec[i] := Dat[PX][PY].V[O][i];
end;

procedure TWaveSim.LoadToTex;
var
 x,y : word;
 p   : PBuffer;
 c   : cardinal;
 S   : Single;
begin
 if (Length(_FarbScala.Table) = 0) then exit;
 Buf[N].Lock(0,Width * Height * sizeof(TVertex),p,0);
 c := 0;
 for x := 0 to Width - 1 do for y := 0 to Height - 1 do begin
  S := Dat[X][Y].Height * _Settings.HFactor;
  p^[y * Width + x].z := - S;
  while (c < Length(_FarbScala.Table) - 1) and (_FarbScala.Table[c].S < S) do inc(c);
  while (c > 0) and ((c >= Length(_FarbScala.Table)) or (_FarbScala.Table[c].S > S)) do dec(c);
  if (c = 0) and (_FarbScala.Table[0].S > s) then begin
   PByte(@p^[y * Width + x].diffuse+0)^ := PByte(@_FarbScala.Table[0].C+2)^;
   PByte(@p^[y * Width + x].diffuse+1)^ := PByte(@_FarbScala.Table[0].C+1)^;
   PByte(@p^[y * Width + x].diffuse+2)^ := PByte(@_FarbScala.Table[0].C+0)^;
  end else if (c = Length(_FarbScala.Table) - 1) then begin
   PByte(@p^[y * Width + x].diffuse+0)^ := PByte(@_FarbScala.Table[c].C+2)^;
   PByte(@p^[y * Width + x].diffuse+1)^ := PByte(@_FarbScala.Table[c].C+1)^;
   PByte(@p^[y * Width + x].diffuse+2)^ := PByte(@_FarbScala.Table[c].C+0)^;
  end else begin
   if Boolean(_FarbScala.Inf and FarbScalaInfBlur) then S := (S - _FarbScala.Table[C].S) / (_FarbScala.Table[C+1].S - _FarbScala.Table[C].S)
   else S := 0;
   PByte(@p^[y * Width + x].diffuse+0)^ := Byte(Round(PByte(@_FarbScala.Table[c].C+2)^ * (1-s) + PByte(@_FarbScala.Table[c+1].C+2)^ * s));
   PByte(@p^[y * Width + x].diffuse+1)^ := Byte(Round(PByte(@_FarbScala.Table[c].C+1)^ * (1-s) + PByte(@_FarbScala.Table[c+1].C+1)^ * s));
   PByte(@p^[y * Width + x].diffuse+2)^ := Byte(Round(PByte(@_FarbScala.Table[c].C+0)^ * (1-s) + PByte(@_FarbScala.Table[c+1].C+0)^ * s));
  end;
  if Boolean(_FarbScala.Inf and FarbScalaInf0Transp) and (Dat[X][Y].Height = 0) then PByte(@p^[y * Width + x].diffuse+3)^ := 0
  else PByte(@p^[y * Width + x].diffuse+3)^ := 255;
 end;
 Buf[N].UnLock;
 CurrentBuf := Buf[N];
 CurrentInd := Ind[N];
end;

procedure TWaveSim.Free;
begin
 DoClose := true;
 if (Working) then begin
  BreakCalc;
  exit;
 end;
 inherited;
end;

destructor TWaveSim.Destroy;
var i : integer;
begin
 DoClose := true;
 for i := 0 to Length(Threads) - 1 do begin Threads[i].Terminate; Threads[i].Resume; end;
 Sleep(10);
 DeleteCriticalSection(SecGetNewWork);
 CurrentBuf := nil;
 CurrentInd := nil;
 Buf[0] := nil;
 Buf[1] := nil;
 Ind[0] := nil;
 Ind[1] := nil;
 inherited;
end;





constructor TWaveSim_Calc.Create(_Parent : TWaveSim);
begin
 FreeOnTerminate := true;
 inherited Create(true);
 TWorking := false;
 Parent := _Parent;
end;

procedure TWaveSim_Calc.WorkRow(Row : Word);
begin
 X := Row + Parent.Offset mod 2;
 Y := Parent.Offset div 2;
 TWorking := true;
end;

procedure TWaveSim_Calc.Execute;
var
 i,j,k       : integer;
 s1,s2,s3,s4 : single;
 C0,C1,C2,C3 : integer;
 B           : Byte;

 Long  : Byte;
 Dir   : integer;
 Dir0  : Word;
 Phi   : integer;
 Amp   : Single;
 Slow  : Single;
 Bx,By : Single;

 // Aufsplittung
 Diff  : Array [0..3] of Single;
 Sum1  : Cardinal;
 Sum2  : Cardinal;
 Sum3  : Cardinal;
 Sum4  : Cardinal;


 Reflect  : Boolean;
 RAmp     : Single;
 RDir     : integer;

 Box   : PWaveSim_Field;
 Src   : PWaveSim_Source;
 _X,_Y : Word;

 // Fast Trig
 fsin,fcin   : word;
 fsout,fcout : smallint;
 fargu,fargo : word;
 fsu,fso     : word;
 fcu,fco     : word;
 fsm,fcm     : single;
 fargm       : single;
 fargxin     : Single;
 fargyin     : Single;

 fargout     : word;
 ttable      : Array of word;
 stable      : Array of smallint;
 ctable      : Array of smallint;

 // NewVec   oder    Cash für Glättung
 NVx,NVy : Word;
 NVi     : integer;
 NV      : PWaveSim_Vector;

label GotoReflect;
label GotoNoSpread;

procedure _cos;
begin
 fcm := fcin / 65535 * (TrigRes-1);
 fcu := Floor(fcm);
 fco := Ceil(fcm);
 if (fcu = fco) then begin
  if (fco > TrigRes div 2) then dec(fcu)
  else inc(fco);
 end;
 fcout := Round(ctable[fco] * (1-fco+fcm) + ctable[fcu] * (1-fcm+fcu));
end;

procedure _sin;
begin
 fsm := fsin / 65535 * (TrigRes-1);
 fsu := Floor(fsm);
 fso := Ceil(fsm);
 if (fsu = fso) then begin
  if (fso > TrigRes div 2) then dec(fsu)
  else inc(fso);
 end;
 fsout := Round(stable[fso] * (1-fso+fsm) + stable[fsu] * (1-fsm+fsu));
end;

procedure _arg;
begin
 fargout := 0;
 if (fargxin > 0) then begin
  fargm := fargyin/fargxin / 40 * TrigRes;
  if (fargm >= 0) then begin
   fargu := floor(fargm);
   fargo := ceil(fargm);
   if (fargu = fargo) then inc(fargo);
   if (fargo >= TrigRes) then begin fargo := TrigRes-1; fargu := TrigRes-2; fargm := fargo; end;
   fargout := Round((ttable[fargo] * (1-fargo+fargm) + ttable[fargu] * (1-fargm+fargu)) / 4);
  end else begin
   fargm := abs(fargm);
   fargu := floor(fargm);
   fargo := ceil(fargm);
   if (fargu = fargo) then inc(fargo);
   if (fargo >= TrigRes) then begin fargo := TrigRes-1; fargu := TrigRes-2; fargm := fargo; end;
   fargout := 65535 - Round((ttable[fargo] * (1-fargo+fargm) + ttable[fargu] * (1-fargm+fargu)) / 4);
  end;
 end else if (fargxin < 0) then begin
  fargm := fargyin/fargxin / 40 * TrigRes;
  if (fargm >= 0) then begin
   fargu := floor(fargm);
   fargo := ceil(fargm);
   if (fargu = fargo) then inc(fargo);
   if (fargo >= TrigRes) then begin fargo := TrigRes-1; fargu := TrigRes-2; fargm := fargo; end;
   fargout := Round((ttable[fargo] * (1-fargo+fargm) + ttable[fargu] * (1-fargm+fargu)) / 4) + 32767;
  end else begin
   fargm := abs(fargm);
   fargu := floor(fargm);
   fargo := ceil(fargm);
   if (fargu = fargo) then inc(fargo);
   if (fargo >= TrigRes) then begin fargo := TrigRes-1; fargu := TrigRes-2; fargm := fargo; end;
   fargout := 32767 - Round((ttable[fargo] * (1-fargo+fargm) + ttable[fargu] * (1-fargm+fargu)) / 4);
  end;
 end else if (fargyin > 0) then fargout := 16384
 else if (fargyin < 0) then fargout := 49151
 else fargout := 0;
end;

procedure NewVec;
begin with (Parent) do begin
 NVi := 0;
 if (NVx < 0) or (NVy < 0) or (NVx >= Width) or (NVy >= Height) then begin NV := @UnusedVector; exit; end;
 while (NVi < Length(Dat[NVx][NVy].V[N])) do begin
  if (not Dat[NVx][NVy].V[N][NVi].Used) then begin
   NV := @Dat[NVx][NVy].V[N][NVi];
   NV^.Used := true;
   exit;
  end;
  inc(NVi);
 end;
 NVi := Length(Dat[NVx][NVy].V[N]);
 SetLength(Dat[NVx][NVy].V[N],NVi + LengthOfVecBuf);
 NV := @Dat[NVx][NVy].V[N][NVi];
 while (NVi < Length(Dat[NVx][NVy].V[N])) do begin
  Dat[NVx][NVy].V[N][NVi].Used := false;
  inc(NVi);
 end;
 NV^.Used := true;
end; end;

begin
 SetLength(ttable,TrigRes);
 SetLength(stable,TrigRes);
 SetLength(ctable,TrigRes);
 for i := 0 to TrigRes-1 do begin
  ttable[i] := Round(arctan(i / TrigRes * 40) * 41720.87678);
  stable[i] := Round(sin(i / TrigRes * 2 * pi) * 32767);
  ctable[i] := Round(cos(i / TrigRes * 2 * pi) * 32767);
 end;
 repeat begin
  while (TWorking) and (Y < Parent.Height-2) do with (Parent) do begin

   // Calc Color of Fields
   B := 0;
   while (B < 4) do begin
    _x := x + B mod 2;
    _y := y + B div 2;
    if (_Settings.ShowType = 0) then begin    // Wellen
     i := 0;
     for j := 0 to Length(Dat[_x][_y].V[O])-1 do begin
      if (Dat[_x][_y].V[O][j].Used) and ((Dat[_x][_y].V[O][j].Dir <= _Settings.BlendoutA) or (Dat[_x][_y].V[O][j].Dir >= _Settings.BlendoutB)) then begin
       fsin := Dat[_x][_y].V[O][j].Phi;
       _sin;
       i := i + Round(fsout / 32767 * Dat[_x][_y].V[O][j].Amp);
      end;
     end;
     case (B) of
      0 : C0 := i;
      1 : C1 := i;
      2 : C2 := i;
      3 : C3 := i;
     end;
    end else if (_Settings.ShowType = 1) then begin    // Speicherauslastung
     i := 0;
     for j := 0 to Length(Dat[_x][_y].V[O])-1 do begin
      i := i + 262140 div ShownWaveAmp;
     end;
     if (i > 255) then i := 255;
     Dat[_x][_y].Height := i;
    end else if (_Settings.ShowType = 2) then begin   // Signifikante Felder
     i := 0;
     for j := 0 to Length(Dat[_x][_y].V[O])-1 do if (Dat[_x][_y].V[O][j].Used) then begin
      i := i + 262140 div ShownWaveAmp;
     end;
     if (i > 255) then i := 255;
     Dat[_x][_y].Height := i;
    end;
    inc(B);
   end;
   if (_Settings.ShowType = 0) then begin
    if (ShowBlur) then begin
     i :=     C0 div 3 * 2; C0 := C0 div 3;
     i := i + C1 div 3 * 2; C1 := C1 div 3;
     i := i + C2 div 3 * 2; C2 := C2 div 3;
     i := i + C3 div 3 * 2; C3 := C3 div 3;
     i := i div 4;
     C0 := C0 + i;
     C1 := C1 + i;
     C2 := C2 + i;
     C3 := C3 + i;
    end;
    Dat[x]  [y].Height   := C0;
    Dat[x+1][y].Height   := C1;
    Dat[x]  [y+1].Height := C2;
    Dat[x+1][y+1].Height := C3;
   end;

   // Move
   repeat begin
    B := 0;
    Long := 0;
    Slow := 0;
    Diff[0] := 0;
    Diff[1] := 0;
    Diff[2] := 0;
    Diff[3] := 0;
    while (B < 4) do begin
     _x := x + B mod 2;
     _y := y + B div 2;
     Box := @Dat[_x][_y];
     Slow := Slow + (1 - Box^.Slow) * 0.25;
     for i := 0 to Length(Box^.V[O])-1 do if (Box^.V[O][i].Used) and (Box^.V[O][i].Amp > 10) then begin
      if (Long = 0) then begin
       // New Center
       Long := Box^.V[O][i].Long;
       Amp := Box^.V[O][i].Amp;
       Dir := Box^.V[O][i].Dir;
       Dir0 := Box^.V[O][i].Dir;
       Phi := Box^.V[O][i].Phi;
       Bx := _x;
       By := _y;
       Box^.V[O][i].Used := false;
      end else if (Long = Box^.V[O][i].Long) and (Box^.V[O][i].Used) and (Dir0 >= integer(Box^.V[O][i].Dir - DirImprecision)) and (Dir0 <= integer(Box^.V[O][i].Dir + DirImprecision)) then begin
       k := Box^.V[O][i].Dir - Dir;
       if (k >  32767) then k := k - 65536;
       if (k < -32767) then k := k + 65536;
       Diff[B] := Diff[B] + k;

       // calc position of center
       S1 := Bx;
       S2 := By;
       Bx := (Bx * Amp + _x * Box^.V[O][i].Amp) / (Amp + Box^.V[O][i].Amp);
       By := (By * Amp + _y * Box^.V[O][i].Amp) / (Amp + Box^.V[O][i].Amp);

       // adjust phase shift
       fargxin := S1 - Bx;
       fargyin := By - S2;
       _arg;
       j := (Round(Dir - fargout) + 65536) mod 65536;;
       fcin := j; _cos;
       Phi := Round(Phi + (sqrt(fargxin*fargxin + fargyin*fargyin) * fcout/32767) / Long * (1-Box^.Slow) * 65535 + 65536) mod 65536;
       fargxin := _x - Bx;
       fargyin := By - _y;
       _arg;
       j := Round(Box^.V[O][i].Dir - fargout + 65536) mod 65536;
       fcin := j; _cos;
       j := Round(Box^.V[O][i].Phi + (sqrt(fargxin*fargxin + fargyin*fargyin) * fcout/32767) / Long * (1-Box^.Slow) * 65535 + 65536) mod 65536;
       Dir := Round((Dir * Amp + Box^.V[O][i].Dir * Box^.V[O][i].Amp) / (Amp + Box^.V[O][i].Amp));

       // Komplex Vector Addition
       fcin := Round(Phi); _cos;
       S1 := fcout/32767 * Amp;
       fcin := Round(j); _cos;
       S1 := S1 + fcout/32767 * Box^.V[O][i].Amp;
       fsin := Round(Phi); _sin;
       S2 := fsout/32767 * Amp;
       fsin := Round(j); _sin;
       S2 := S2 + fsout/32767 * Box^.V[O][i].Amp;
       Amp := Sqrt(S1*s1 + S2*S2);
       fargxin := S1;
       fargyin := S2;
       _arg;
       Phi := fargout;
       Box^.V[O][i].Used := false;
      end;
     end;
     inc(B);
    end;
    if (Long <> 0) and (Amp > MinAmp) then begin

     // Search parallel vectors
     Diff[0] := Diff[0] / 3;
     Diff[1] := Diff[1] / 3;
     Diff[2] := Diff[2] / 3;
     Diff[3] := Diff[3] / 3;
     if (WavelikeSpread > 16) or (Count mod WavelikeSpread <> 0) then goto GotoNoSpread;
     Sum1 := 0;
     Sum2 := 0;
     Sum3 := 0;
     Sum4 := 0;
     if (x > 0) and (x < Width - 3) and (y > 0) and (y < Height - 3) then while (B < 16) do begin
    //  while (B = 5) or (B = 6) or (B = 9) or (B = 10) do inc(B);
      _x := x + B mod 4 - 1;
      _y := y + B div 4 - 1;
      Box := @Dat[_x][_y];
      for i := 0 to Length(Box^.V[O])-1 do begin
       C0 := Box^.V[O][i].Dir - Dir;
       if (C0 >  32767) then C0 := C0 - 65536;
       if (C0 < -32767) then C0 := C0 + 65536;
       if (Box^.V[O][i].Long = Long) and (C0 < 8192) and (C0 > -8192) and (Box^.V[O][i].Amp > 10) then begin

        if (_x < Bx) and (_y < By) then Sum1 := Sum1 + Box^.V[O][i].Amp
        else if (_x > Bx) and (_y < By) then Sum2 := Sum2 + Box^.V[O][i].Amp
        else if (_x < Bx) and (_y > By) then Sum3 := Sum3 + Box^.V[O][i].Amp
        else if (_x > Bx) and (_y > By) then Sum4 := Sum4 + Box^.V[O][i].Amp;

    {    k := Box^.V[O][i].Phi - Phi;
        if (k >  32767) then k := k - 65536;
        if (k < -32767) then k := k + 65536;
        S3 := k * Long / (1-Box^.Slow);
        fsin := Box^.V[O][i].Dir; _sin;
        fcin := Box^.V[O][i].Dir; _cos;
        S1 := _x + S3 * fcout / 32767;
        S2 := _y - S3 * fsout / 32767;
        fargxin := (S1 - Bx);
        fargyin := (By - S2);
        _arg;
        j := fargout - Dir;
        if (j >  32767) then j := j - 65536;
        if (j < -32767) then j := j + 65536;
        if (j < 0) then j := j + 16384
        else j := j - 16384;
        j := j div 10;
        // j beugt die Vektoren zusammen, wenn die Wellen vor dem Mittelpunkt sind und auseinander, wenn sie dahinter sind

        S1 := 1 / (abs(C0 / 8192) + 1) * 1 / (abs(S3) / 30 + 1);
        k := C0;

   {     fargxin := (_x - Bx);
        fargyin := (By - _y);
        _arg;
        j := fargout - Dir;
        if (j > 0) then j := j - 16384
        else j := j + 16384;
        if (j >  32767) then j := j - 65536;
        if (j < -32767) then j := j + 65536;
        fcin := k; _cos;
        k := Round(Phi + (sqrt(fargxin*fargxin + fargyin*fargyin) * fcout/32767) / Long * (1-Box^.Slow) * 65535 + 65536) mod 65536;
        S1 := 1 - ((k - Box^.V[O][i].Phi)*(k - Box^.V[O][i].Phi)*0.00000001490116119);
        k := (k - Box^.V[O][i].Phi);
        k := 0;
        k := Box^.V[O][i].Dir - Dir;
        if (k >  32767) then k := k - 65536;
        if (k < -32767) then k := k + 65536;
        S4 := 1 / ((Box^.V[O][i].Amp - Amp/4) * (Box^.V[O][i].Amp - Amp/4) * 0.0015 + 1) * 65535;     }

        S2 := 1 / ((_x - x) * (_x - x) + (_y - y) * (_y - y));
        S3 := S4 * S1 * S2;
        if (S3 > 0) then begin
         Diff[0] := (Diff[0] * Sum1 + (j+k) * S3) / (Sum1 + S3);
         Sum1 := Round(Sum1 + S3);
        end;

        S2 := 1 / ((_x - (x+1)) * (_x - (x+1)) + (_y - y) * (_y - y));
        S3 := S4 * S1 * S2;
        if (S3 > 0) then begin
         Diff[1] := (Diff[1] * Sum2 + (j+k) * S3) / (Sum2 + S3);
         Sum2 := Round(Sum2 + S3);
        end;

        S2 := 1 / ((_x - x) * (_x - x) + (_y - (y+1)) * (_y - (y+1)));
        S3 := S4 * S1 * S2;
        if (S3 > 0) then begin
         Diff[2] := (Diff[2] * Sum3 + (j+k) * S3) / (Sum3 + S3);
         Sum3 := Round(Sum3 + S3);
        end;

        S2 := 1 / ((_x - (x+1)) * (_x - (x+1)) + (_y - (y+1)) * (_y - (y+1)));
        S3 := S4 * S1 * S2;
        if (S3 > 0) then begin
         Diff[3] := (Diff[3] * Sum4 + (j+k) * S3) / (Sum4 + S3);
         Sum4 := Round(Sum4 + S3);
        end;        }

       end;
      end;
      inc(B);
     end;
     if (Sum1+Sum2+Sum3+Sum4 > 0) then begin
      S1 := ln(abs(Dir mod 16384 - 8192) / 8192 * 2 + 1);
      k := 24567 - Dir;
      while (k < -32767) do k := k + 65536;
      while (k >  32767) do k := k - 65536;
      S2 := Sum1 / (Sum1+Sum2+Sum3+Sum4) * 0.2;
      Diff[0] := Diff[0] + k / (100 + S1*S1*150) * 0.4 - k * S2 * S2; //Sum1 * (Random-0.5) * abs(Dir mod 16384 - 8192) / 8192 / 10 / (Sum1+Sum2+Sum3+Sum4);
      k := 8192 - Dir;
      while (k < -32767) do k := k + 65536;
      while (k >  32767) do k := k - 65536;
      S2 := Sum2 / (Sum1+Sum2+Sum3+Sum4) * 0.2;
      Diff[1] := Diff[1] + k / (100 + S1*S1*150) * 0.4 - k * S2 * S2; //Sum2 * (Random-0.5) * abs(Dir mod 16384 - 8192) / 10 / (Sum1+Sum2+Sum3+Sum4);
      k := 40960 - Dir;
      while (k < -32767) do k := k + 65536;
      while (k >  32767) do k := k - 65536;
      S2 := Sum3 / (Sum1+Sum2+Sum3+Sum4) * 0.2;
      Diff[2] := Diff[2] + k / (100 + S1*S1*150) * 0.4 - k * S2 * S2; //Sum3 * (Random-0.5) * abs(Dir mod 16384 - 8192) / 10 / (Sum1+Sum2+Sum3+Sum4);
      k := 57344 - Dir;
      while (k < -32767) do k := k + 65536;
      while (k >  32767) do k := k - 65536;
      S2 := Sum4 / (Sum1+Sum2+Sum3+Sum4) * 0.2;
      Diff[3] := Diff[3] + k / (100 + S1*S1*150) * 0.4 - k * S2 * S2; //Sum4 * (Random-0.5) * abs(Dir mod 16384 - 8192) / 10 / (Sum1+Sum2+Sum3+Sum4);
     end;
     GotoNoSpread:

     // Move Center
     fcin := Round(Dir); _cos;
     fsin := Round(Dir); _sin;
     Bx := Bx + fcout/32767 * Slow;
     By := By - fsout/32767 * Slow;

     NVx := Round(Bx);
     NVy := Round(By);
     Reflect := false;
     if (NVx < Width) and (NVy < Height) then begin
      // Absorb
      if (Dat[NVx][NVy].Absorb > 0) then Amp := Round(Amp * (1 - Dat[NVx][NVy].Absorb));
      // Reflect
      if (Dat[NVx][NVy].Reflect > 0) then begin
       RAmp := Round(Amp * (1 - Dat[NVx][NVy].Reflect));
       RDir := Dir;
       Amp := Round(Amp * Dat[NVx][NVy].Reflect);
       Dir := (Dir + 32767 - 2 * (Dir - Dat[NVx][NVy].Normale) + 131072) mod 65536;
       Reflect := true;
      end;
     end;
     GotoReflect:

     // calc circle parts
     if (Bx >= 0) and (By >= 0) then begin
      NVx := Floor(Bx);
      NVy := Floor(By);
      i := Round((1 - By + NVy) * (SquareResolution-1)) * SquareResolution + Round((Bx - NVx) * (SquareResolution-1));
      C0 := Round(CPA[i] / 65535 * Amp);
     end else C0 := -1;
     if (Bx >= 0) and (By < Height-1) then begin
      NVx := Floor(Bx);
      NVy := Ceil(By);
      i := Round((By - Floor(By)) * (SquareResolution-1)) * SquareResolution + Round((Bx - NVx) * (SquareResolution-1));
      C1 := Round(CPA[i] / 65535 * Amp);
     end else C1 := -1;
     if (Bx < Width-1) and (By < Height-1) then begin
      NVx := Ceil(Bx);
      NVy := Ceil(By);
      i := Round((By - Floor(By)) * (SquareResolution-1)) * SquareResolution + Round((1 - Bx + Floor(Bx)) * (SquareResolution-1));
      C2 := Round(CPA[i] / 65535 * Amp);
     end else C2 := -1;
     if (Bx < Width-1) and (By >= 0) then begin
      NVx := Ceil(Bx);
      NVy := Floor(By);
      i := Round((1 - By + NVy) * (SquareResolution-1)) * SquareResolution + Round((1 - Bx + Floor(Bx)) * (SquareResolution-1));
      C3 := Round(CPA[i] / 65535 * Amp);
     end else C3 := -1;

     // blur amplitudes
     S2 := 1 / (2.5 - abs(Dir mod 16384 - 8192) / 8192);
     S3 := 0.8 * S2 * S2;
     S3 := (S3 + 1 - 1 / ((abs(Diff[0]) + abs(Diff[1]) + abs(Diff[2]) + abs(Diff[3])) / 200 + 1)) / 2;
     j := 0;
     i := Round(C0 * S3);
     C0 := C0 - i;
     j := j + i;
     i := Round(C1 * S3);
     C1 := C1 - i;
     j := j + i;
     i := Round(C2 * S3);
     C2 := C2 - i;
     j := j + i;
     i := Round(C3 * S3);
     C3 := C3 - i;
     j := j + i;
     i := j div 4;
     C0 := C0 + i;
     C1 := C1 + i;
     C2 := C2 + i;
     C3 := C3 + i;

     // Fill circle parts with adjusted phase shifts
     if (C0 > 0) and (Bx >= 0) and (By >= 0) then begin
      NVx := Floor(Bx);
      NVy := Floor(By);
      NewVec;
      fargxin := -NVx + Bx;
      fargyin := -By + NVy;
      _arg;
      k := Round(Dir + Diff[0] + 65536) mod 65536;
      j := (k - fargout + 131072) mod 65536;
      fcin := j; _cos;
      j := Round(Phi + (sqrt(fargxin*fargxin + fargyin*fargyin) * fcout/32767) / Long * (1-Box^.Slow) * 65535 + 65536) mod 65536;
      NV^.Phi := j;
      NV^.Amp := C0;
      NV^.Dir := k;
      NV^.Long := Long;
     end else C0 := -1;
     if (C1 > 0) and (Bx >= 0) and (By < Height-1) then begin
      NVx := Floor(Bx);
      NVy := Ceil(By);
      if (NVy = Floor(By)) then inc(NVy);
      NewVec;
      fargxin := -NVx + Bx;
      fargyin := -By + NVy;
      _arg;
      k := Round(Dir + Diff[2] + 65536) mod 65536;
      j := (k - fargout + 131072) mod 65536;
      fcin := j; _cos;
      j := Round(Phi + (sqrt(fargxin*fargxin + fargyin*fargyin) * fcout/32767) / Long * (1-Box^.Slow) * 65535 + 65536) mod 65536;
      NV^.Phi := j;
      NV^.Amp := C1;
      NV^.Dir := k;
      NV^.Long := Long;
     end else C1 := -1;
     if (C2 > 0) and (Bx < Width-1) and (By < Height-1) then begin
      NVx := Ceil(Bx);
      NVy := Ceil(By);
      if (NVx = Floor(Bx)) then inc(NVx);
      if (NVy = Floor(By)) then inc(NVy);
      NewVec;
      fargxin := -NVx + Bx;
      fargyin := -By + NVy;
      _arg;
      k := Round(Dir + Diff[3] + 65536) mod 65536;
      j := (k - fargout + 131072) mod 65536;
      fcin := j; _cos;
      j := Round(Phi + (sqrt(fargxin*fargxin + fargyin*fargyin) * fcout/32767) / Long * (1-Box^.Slow) * 65535 + 65536) mod 65536;
      NV^.Phi := j;
      NV^.Amp := C2;
      NV^.Dir := k;
      NV^.Long := Long;
     end else C2 := -1;
     if (C3 > 0) and (Bx < Width-1) and (By >= 0) then begin
      NVx := Ceil(Bx);
      NVy := Floor(By);
      if (NVx = Floor(Bx)) then inc(NVx);
      NewVec;
      fargxin := -NVx + Bx;
      fargyin := -By + NVy;
      _arg;
      k := Round(Dir + Diff[1] + 65536) mod 65536;
      j := (k - fargout + 131072) mod 65536;
      fcin := j; _cos;
      j := Round(Phi + (sqrt(fargxin*fargxin + fargyin*fargyin) * fcout/32767) / Long * (1-Box^.Slow) * 65535 + 65536) mod 65536;
      NV^.Phi := j;
      NV^.Amp := C3;
      NV^.Dir := k;
      NV^.Long := Long;
     end else C3 := -1;

     // Repeat if reflect
     if (Reflect) then begin
      Reflect := false;
      Amp := RAmp;
      Dir := RDir;
      goto GotoReflect;
     end;

    end;
   end until (Long = 0);

   // Add sources' vectors
   B := 0;
   while (B < 4) do begin
    _x := x + B mod 2;
    _y := y + B div 2;
    Box := @Dat[_x][_y];
    for i := 0 to Length(Box^.V[O])-1 do begin
     Box^.V[O][i].Used := false;
    end;
    for i := Length(Box^.V[N])-1 downto 0 do begin
     if (Box^.V[N][i].Used) then break
     else Box^.V[N][i].Long := 0;
    end;
    if (Dat[_x][_y].Source > 0) then begin
     Src := @Sources[Dat[_x][_y].Source-1];
     for j := 0 to Length(Src^.Fourier)-1 do case (Src^.Art) of
      SrcCenter : begin
       NVx := _x;
       NVy := _y;
       for i := 0 to CircleSourceN do begin
        NewVec;
        NV^.Long := Src^.Fourier[j].Long;
        NV^.Dir := Round(i / CircleSourceN * 65535 + Random * 65536 / CircleSourceN) mod 65536;
        NV^.Amp := Src^.Fourier[j].Amp;
        NV^.Phi := Src^.Fourier[j].Phi;
       end;
      end;
      SrcLaser : begin
       NVx := _x;
       NVy := _y;
       NewVec;
       NV^.Long := Src^.Fourier[j].Long;
       NV^.Dir := Src^.Dir;
       NV^.Amp := Src^.Fourier[j].Amp;
       NV^.Phi := Src^.Fourier[j].Phi;
      end;
     end;
    end;
    inc(B);
   end;

   // Delete unused Vectors
   if (Count mod 20 = 0) then for k := 0 to 1 do for B := 0 to 3 do begin
    _x := x + B mod 2;
    _y := y + B div 2;
    for i := Length(Dat[_x][_y].V[k])-1 downto 1 do begin
     if (Dat[_x][_y].V[k][i-1].Long <> 0) then break;
    end;
    if (Length(Dat[_x][_y].V[k]) > 0) then begin
     if (i = 1) and (Dat[_x][_y].V[k][0].Long = 0) then i := 0;
     j := 0;
     while (j < i) do j := j + LengthOfVecBuf;
     if (Length(Dat[_x][_y].V[k]) <> j) then SetLength(Dat[_x][_y].V[k],j);
    end;
   end;
   if (parent.ReduceCalculationCapacity) and (Self = parent.Threads[Length(parent.Threads)-1]) and (Y mod 20 = 0) then Sleep(10);
   inc(Y,2);
  end;

  EnterCriticalSection(Parent.SecGetNewWork);
  TWorking := false;
  Parent.GetNewWork(self);
  for i := 0 to Length(Parent.Threads) - 1 do if (Parent.Threads[i].TWorking) then break;
  if (not Parent.Threads[i].TWorking) then begin
   if (Parent.Switched) then Synchronize(@Parent.EndCalc)
   else Synchronize(@Parent.Switch);
  end;
  LeaveCriticalSection(Parent.SecGetNewWork);
  if (not TWorking) then Suspend;
 end until (Terminated);
end;

end.


