unit WaveSimulationTools;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Windows, Forms, WaveSimulation, Controls, StdCtrls,
  ExtCtrls, direct3D9, d3dx9, graph2D, Math;

type
 TOnCloseAction = procedure(Sender : TObject; var _Action : TCloseAction) of object;
 TVectorPointer = class(TForm)
  private
   i         : integer;
   b         : byte;
   S1        : Single;
   Sim       : TWaveSim;
   Cont      : TWinControl;
   D3DDev9   : IDirect3DDevice9;
   _OnClose  : TOnCloseAction;
   ButSetPos : TButton;
   LabelPos  : TLabel;
   Pos       : TPoint;
   Vec       : TWaveSim_Vectors;
   Buf       : Array of IDirect3DVertexbuffer9;
   PBuf      : Pointer;
   procedure FormClose(Sender : TObject; var _Action : TCloseAction);
   procedure ButSetPosClick(Sender : TObject);
   procedure PanelMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
   procedure PanelMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  public
   constructor Create(_Sim : TWaveSim; _Cont : TWinControl; _D3DDev9 : IDirect3DDevice9);
   procedure Render;
   property OnClose : TOnCloseAction read _OnClose write _OnClose;
   destructor Destroy; override;
 end;




 TFourierGraph = class(TForm)
  private
   i,_j      : integer;
   b         : byte;
   S1        : Single;
   Sim       : TWaveSim;
   Cont      : TWinControl;
   D3DDev9   : IDirect3DDevice9;
   _OnClose  : TOnCloseAction;
   ButSetPos : TButton;
   LabelPos  : TLabel;
   Pos       : TPoint;
   Vec       : TWaveSim_Vectors;
   Buf       : IDirect3DVertexbuffer9;
   PBuf      : Pointer;
   Graph     : TGraph2D;
   procedure FormClose(Sender : TObject; var _Action : TCloseAction);
   procedure ButSetPosClick(Sender : TObject);
   procedure PanelMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
   procedure PanelMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  public
   constructor Create(_Sim : TWaveSim; _Cont : TWinControl; _D3DDev9 : IDirect3DDevice9);
   procedure Render;
   property OnClose : TOnCloseAction read _OnClose write _OnClose;
   destructor Destroy; override;
 end;





 T2DGraph = class(TForm)
  private
   i,_j      : integer;
   e         : extended;
   b         : byte;
   S1        : Single;
   Sim       : TWaveSim;
   Cont      : TWinControl;
   D3DDev9   : IDirect3DDevice9;
   _OnClose  : TOnCloseAction;
   ButSetPos : TButton;
   LabelPos1 : TLabel;
   LabelPos2 : TLabel;
   Pos1      : TPoint;
   Pos2      : TPoint;
   PosClick  : Byte;
   Vec       : TWaveSim_Vectors;
   Buf       : IDirect3DVertexbuffer9;
   PBuf      : Pointer;
   Graph     : TGraph2D;
   procedure FormClose(Sender : TObject; var _Action : TCloseAction);
   procedure ButSetPosClick(Sender : TObject);
   procedure PanelMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
   procedure PanelMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  public
   constructor Create(_Sim : TWaveSim; _Cont : TWinControl; _D3DDev9 : IDirect3DDevice9);
   procedure Render;
   property OnClose : TOnCloseAction read _OnClose write _OnClose;
   destructor Destroy; override;
 end;



var
 VecPointerWidth : single = 0.01;
 VecPointerSize  : single = 0.5;
 Is3DOn          : Boolean = false;

implementation

type
 TVectorPointer_Vertex = packed record
  x,y,z : Single;
  color : Cardinal;
 end;
 TBuffer = Array [0..1] of TVectorPointer_Vertex;
 PBuffer = ^TBuffer;

procedure ControlSetMouseMove(C : TWinControl; P : TMouseMoveEvent);
begin
 if (C is TForm) then TForm(c).OnMouseMove := P
 else if (C is TPanel) then TPanel(c).OnMouseMove := P;
end;

function ControlGetMouseMove(C : TWinControl) : TMouseMoveEvent;
begin
 if (C is TForm) then Result := TForm(c).OnMouseMove
 else if (C is TPanel) then Result := TPanel(c).OnMouseMove
 else Result := nil;
end;

procedure ControlSetMouseDown(C : TWinControl; P : TMouseEvent);
begin
 if (C is TForm) then TForm(c).OnMouseDown := P
 else if (C is TPanel) then TPanel(c).OnMouseDown := P;
end;

function ControlGetMouseDown(C : TWinControl) : TMouseEvent;
begin
 if (C is TForm) then Result := TForm(c).OnMouseDown
 else if (C is TPanel) then Result := TPanel(c).OnMouseDown
 else Result := nil;
end;

constructor TVectorPointer.Create(_Sim : TWaveSim; _Cont : TWinControl; _D3DDev9 : IDirect3DDevice9);
begin
 inherited Create(nil);
 Sim := _Sim;
 Cont := _Cont;
 D3DDev9 := _D3DDev9;
 TForm(self).OnClose := @FormClose;
 _OnClose := nil;
 Pos.X := 0;
 Pos.Y := 0;

 self.BorderStyle := bsToolWindow;
 self.Caption := 'Vektor Zeiger';
 self.width := 200;
 self.height := 50;
 self.Top := 30;
 self.left := 30;

 ButSetPos := TButton.Create(Self);
 ButSetPos.Parent := self;
 ButSetPos.Caption := 'Position setzen';
 ButSetPos.Left := 10;
 ButSetPos.Top := 10;
 ButSetPos.Width := 100;
 ButSetPos.Height := 30;
 ButSetPos.OnClick := @ButSetPosClick;

 LabelPos := TLabel.Create(self);
 LabelPos.Parent := self;
 LabelPos.Left := 120;
 LabelPos.Top := 10;
 LabelPos.Caption := 'X : 0' + #13#10 + 'Y : 0';

 Show;
 ButSetPosClick(self);
end;

procedure TVectorPointer.Render;
begin
 b := 0;
 while (b < 4) do begin
 Sim.CopyVectors(Pos.X + b mod 2,Pos.Y + b div 2,Vec);
 if (Length(Buf) > Length(Vec)) then begin
  i := Length(Buf) - 1;
  while (i >= Length(Vec)) do begin Buf[i] := nil; dec(i); end;
  SetLength(Buf,Length(Vec));
 end else begin
  i := Length(Buf);
  SetLength(Buf,Length(Vec));
  while (i < Length(Buf)) do begin
   D3DDev9.CreateVertexBuffer(SizeOf(TVectorPointer_Vertex) * 5,D3DUSAGE_WRITEONLY,D3DFVF_XYZ or D3DFVF_DIFFUSE,D3DPOOL_MANAGED,Buf[i],nil);
   inc(i);
  end;
 end;
 i := 0;
 while (i < Length(Buf)) do begin
  S1 := Vec[i].Amp / 65536 * VecPointerSize;
  Buf[i].Lock(0,SizeOf(TVectorPointer_Vertex) * 5,PBuf,0);
  PBuffer(PBuf)^[0].X :=  Pos.X / Sim.Width  * 2 - 1 + cos((Vec[i].Dir + 16384) * 0.00009587379924852577) * VecPointerWidth;
  PBuffer(PBuf)^[0].Y := -Pos.Y / Sim.Height * 2 + 1 + sin((Vec[i].Dir + 16384) * 0.00009587379924852577) * VecPointerWidth;
  PBuffer(PBuf)^[0].Z := 0;
  PBuffer(PBuf)^[0].Color := $FFFF00FF;
  PBuffer(PBuf)^[1].X :=  Pos.X / Sim.Width  * 2 - 1 + cos((Vec[i].Dir - 16384) * 0.00009587379924852577) * VecPointerWidth;
  PBuffer(PBuf)^[1].Y := -Pos.Y / Sim.Height * 2 + 1 + sin((Vec[i].Dir - 16384) * 0.00009587379924852577) * VecPointerWidth;
  PBuffer(PBuf)^[1].Z := 0;
  PBuffer(PBuf)^[1].Color := $FFFF00FF;
  PBuffer(PBuf)^[2].X :=  Pos.X / Sim.Width  * 2 - 1 + cos(Vec[i].Dir * 0.00009587379924852577) * S1 + cos((Vec[i].Dir + 16384) * 0.00009587379924852577) * VecPointerWidth;
  PBuffer(PBuf)^[2].Y := -Pos.Y / Sim.Height * 2 + 1 + sin(Vec[i].Dir * 0.00009587379924852577) * S1 + sin((Vec[i].Dir + 16384) * 0.00009587379924852577) * VecPointerWidth;
  PBuffer(PBuf)^[2].Z := 0;
  PBuffer(PBuf)^[2].Color := $FFFF00FF;
  PBuffer(PBuf)^[3].X :=  Pos.X / Sim.Width  * 2 - 1 + cos(Vec[i].Dir * 0.00009587379924852577) * S1 + cos((Vec[i].Dir - 16384) * 0.00009587379924852577) * VecPointerWidth;
  PBuffer(PBuf)^[3].Y := -Pos.Y / Sim.Height * 2 + 1 + sin(Vec[i].Dir * 0.00009587379924852577) * S1 + sin((Vec[i].Dir - 16384) * 0.00009587379924852577) * VecPointerWidth;
  PBuffer(PBuf)^[3].Z := 0;
  PBuffer(PBuf)^[3].Color := $FFFF00FF;
  PBuffer(PBuf)^[4].X :=  Pos.X / Sim.Width  * 2 - 1 + cos(Vec[i].Dir * 0.00009587379924852577) * S1 + cos((Vec[i].Dir) * 0.00009587379924852577) * VecPointerWidth;
  PBuffer(PBuf)^[4].Y := -Pos.Y / Sim.Height * 2 + 1 + sin(Vec[i].Dir * 0.00009587379924852577) * S1 + sin((Vec[i].Dir) * 0.00009587379924852577) * VecPointerWidth;
  PBuffer(PBuf)^[4].Z := 0;
  PBuffer(PBuf)^[4].Color := $FFFF00FF;
  Buf[i].Unlock;
  d3ddev9.SetStreamSource(0,Buf[i],0,SizeOf(TVectorPointer_Vertex));
  d3ddev9.SetFVF(D3DFVF_XYZ or D3DFVF_DIFFUSE);
  d3ddev9.DrawPrimitive(D3DPT_TRIANGLESTRIP,0,3);
  inc(i);
 end;
 inc(b);
 end;
end;

procedure TVectorPointer.ButSetPosClick(Sender : TObject);
begin
 if (Is3DOn) then exit;
 Cont.Cursor := crCross;
 ControlSetMouseMove(Cont,@PanelMouseMove);
 ControlSetMouseDown(Cont,@PanelMouseDown);
end;

procedure TVectorPointer.PanelMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
 Pos.X := Round(X / Cont.Width * Sim.Width);
 Pos.Y := Round(Y / Cont.Height * Sim.Height);
 LabelPos.Caption := 'X : ' + IntToStr(Pos.X) + #13#10 + 'Y : ' + IntToStr(Pos.Y);
end;

procedure TVectorPointer.PanelMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 ControlSetMouseMove(Cont,nil);
 ControlSetMouseDown(Cont,nil);
 PanelMouseMove(Sender,Shift,X,Y);
 Cont.Cursor := crDefault;
end;

procedure TVectorPointer.FormClose(Sender : TObject; var _Action : TCloseAction);
begin
 if (_OnClose <> nil) then _OnClose(self,_Action);
 _Action := caFree;
end;

destructor TVectorPointer.Destroy;
var j : integer;
begin
 if (ControlGetMouseMove(Cont) = @PanelMouseMove) then ControlSetMouseMove(Cont,nil);
 if (ControlGetMouseDown(Cont) = @PanelMouseDown) then ControlSetMouseDown(Cont,nil);
 for j := 0 to Length(Buf)-1 do Buf[j] := nil;
 inherited;
end;












constructor TFourierGraph.Create(_Sim : TWaveSim; _Cont : TWinControl; _D3DDev9 : IDirect3DDevice9);
var j : integer;
begin
 inherited Create(nil);
 Sim := _Sim;
 Cont := _Cont;
 D3DDev9 := _D3DDev9;
 TForm(self).OnClose := @FormClose;
 _OnClose := nil;
 Pos.X := 0;
 Pos.Y := 0;

 self.BorderStyle := bsSizeToolWin;
 self.Caption := 'Fourier Graph';
 self.width := 400;
 self.height := 300;
 self.Top := 30;
 self.left := 30;

 ButSetPos := TButton.Create(Self);
 ButSetPos.Parent := self;
 ButSetPos.Caption := 'Position setzen';
 ButSetPos.Left := 10;
 ButSetPos.Top := 10;
 ButSetPos.Width := 100;
 ButSetPos.Height := 30;
 ButSetPos.OnClick := @ButSetPosClick;

 LabelPos := TLabel.Create(self);
 LabelPos.Parent := self;
 LabelPos.Left := 120;
 LabelPos.Top := 10;
 LabelPos.Caption := 'X : 0' + #13#10 + 'Y : 0';

 Graph := TGraph2D.Create(self);
 Graph.Parent := self;
 Graph.Left := 10;
 Graph.Top := 50;
 Graph.AnchorParallel(akRight,10,self);
 Graph.AnchorParallel(akBottom,10,self);
 Graph.MinX :=  10;
 Graph.MaxX :=  256;
 Graph.MaxY :=  100000;
 Graph.MinY := -10;
 Graph.SetViewArea(10,-10,256,100);

 D3DDev9.CreateVertexBuffer(SizeOf(TVectorPointer_Vertex) * 4,D3DUSAGE_WRITEONLY,D3DFVF_XYZ or D3DFVF_DIFFUSE,D3DPOOL_MANAGED,Buf,nil);
 Buf.Lock(0,SizeOf(TVectorPointer_Vertex) * 5,PBuf,0);
 PBuffer(PBuf)^[0].X :=  Pos.X / Sim.Width  * 2 - 1 - VecPointerWidth;
 PBuffer(PBuf)^[0].Y := -Pos.Y / Sim.Height * 2 + 1 - VecPointerWidth;
 PBuffer(PBuf)^[0].Z := 0;
 PBuffer(PBuf)^[0].Color := $FF00FFFF;
 PBuffer(PBuf)^[1].X :=  Pos.X / Sim.Width  * 2 - 1 - VecPointerWidth;
 PBuffer(PBuf)^[1].Y := -Pos.Y / Sim.Height * 2 + 1 + VecPointerWidth;
 PBuffer(PBuf)^[1].Z := 0;
 PBuffer(PBuf)^[1].Color := $FF00FFFF;
 PBuffer(PBuf)^[2].X :=  Pos.X / Sim.Width  * 2 - 1 + VecPointerWidth;
 PBuffer(PBuf)^[2].Y := -Pos.Y / Sim.Height * 2 + 1 - VecPointerWidth;
 PBuffer(PBuf)^[2].Z := 0;
 PBuffer(PBuf)^[2].Color := $FF00FFFF;
 PBuffer(PBuf)^[3].X :=  Pos.X / Sim.Width  * 2 - 1 + VecPointerWidth;
 PBuffer(PBuf)^[3].Y := -Pos.Y / Sim.Height * 2 + 1 + VecPointerWidth;
 PBuffer(PBuf)^[3].Z := 0;
 PBuffer(PBuf)^[3].Color := $FF00FFFF;
 Buf.Unlock;

 SetLength(Graph.Lines,1);
 SetLength(Graph.Lines[0],247);
 for j := 0 to 246 do begin
  Graph.Lines[0][j].X := j + 10;
  Graph.Lines[0][j].Y := 0;
 end;

 Show;
 ButSetPosClick(self);
end;

procedure TFourierGraph.Render;
begin
 i := 0;
 while (i < 247) do begin
  Graph.Lines[0][i].Y := 0;
  inc(i);
 end;
 B := 0;
 while (B < 4) do begin
  Sim.CopyVectors(Pos.X + B mod 2,Pos.Y + B div 2,Vec);
  i := 0;
  while (i < Length(Vec)) do begin
   _j := Vec[i].Long - 10;
   if (_j >= 0) and (_j < 247) then Graph.Lines[0][_j].Y := Graph.Lines[0][_j].Y + Vec[i].Amp / 65536 * 100;
   inc(i);
  end;
  inc(B);
 end;
 d3ddev9.SetStreamSource(0,Buf,0,SizeOf(TVectorPointer_Vertex));
 d3ddev9.SetFVF(D3DFVF_XYZ or D3DFVF_DIFFUSE);
 d3ddev9.DrawPrimitive(D3DPT_TRIANGLESTRIP,0,2);
 Graph.DoPaint(self);
end;

procedure TFourierGraph.ButSetPosClick(Sender : TObject);
begin
 if (Is3DOn) then exit;
 Cont.Cursor := crCross;
 ControlSetMouseMove(Cont,@PanelMouseMove);
 ControlSetMouseDown(Cont,@PanelMouseDown);
end;

procedure TFourierGraph.PanelMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
 Pos.X := Round(X / Cont.Width * Sim.Width);
 Pos.Y := Round(Y / Cont.Height * Sim.Height);
 LabelPos.Caption := 'X : ' + IntToStr(Pos.X) + #13#10 + 'Y : ' + IntToStr(Pos.Y);
 Buf.Lock(0,SizeOf(TVectorPointer_Vertex) * 5,PBuf,0);
 PBuffer(PBuf)^[0].X :=  Pos.X / Sim.Width  * 2 - 1 - VecPointerWidth;
 PBuffer(PBuf)^[0].Y := -Pos.Y / Sim.Height * 2 + 1 - VecPointerWidth;
 PBuffer(PBuf)^[1].X :=  Pos.X / Sim.Width  * 2 - 1 - VecPointerWidth;
 PBuffer(PBuf)^[1].Y := -Pos.Y / Sim.Height * 2 + 1 + VecPointerWidth;
 PBuffer(PBuf)^[2].X :=  Pos.X / Sim.Width  * 2 - 1 + VecPointerWidth;
 PBuffer(PBuf)^[2].Y := -Pos.Y / Sim.Height * 2 + 1 - VecPointerWidth;
 PBuffer(PBuf)^[3].X :=  Pos.X / Sim.Width  * 2 - 1 + VecPointerWidth;
 PBuffer(PBuf)^[3].Y := -Pos.Y / Sim.Height * 2 + 1 + VecPointerWidth;
 Buf.Unlock;
end;

procedure TFourierGraph.PanelMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 ControlSetMouseMove(Cont,nil);
 ControlSetMouseDown(Cont,nil);
 PanelMouseMove(Sender,Shift,X,Y);
 Cont.Cursor := crDefault;
end;

procedure TFourierGraph.FormClose(Sender : TObject; var _Action : TCloseAction);
begin
 if (_OnClose <> nil) then _OnClose(self,_Action);
 _Action := caFree;
end;

destructor TFourierGraph.Destroy;
var j : integer;
begin
 if (ControlGetMouseMove(Cont) = @PanelMouseMove) then ControlSetMouseMove(Cont,nil);
 if (ControlGetMouseDown(Cont) = @PanelMouseDown) then ControlSetMouseDown(Cont,nil);
 Buf := nil;
 inherited;
end;






function arg(x,y : extended) : extended;
var q : extended;
begin
 q := y/x;
 if (isInfinite(q)) then begin
  if (y > 0) then Result := pi/2
  else Result := pi*1.5;
 end else if (IsNaN(q)) then Result := q
 else if (x > 0) then begin
  if (y >= 0) then Result := arctan(q)
  else Result := 2 * pi + arctan(q);
 end else begin
  if (y < 0) then Result := arctan(q) + pi
  else Result := -pi + arctan(q);
 end;
end;

constructor T2DGraph.Create(_Sim : TWaveSim; _Cont : TWinControl; _D3DDev9 : IDirect3DDevice9);
var j : integer;
begin
 inherited Create(nil);
 Sim := _Sim;
 Cont := _Cont;
 D3DDev9 := _D3DDev9;
 TForm(self).OnClose := @FormClose;
 _OnClose := nil;
 Pos1.X := 0;
 Pos1.Y := 0;
 Pos2.X := 0;
 Pos2.Y := 0;

 self.BorderStyle := bsSizeToolWin;
 self.Caption := '2D Graph';
 self.width := 600;
 self.height := 300;
 self.Top := 30;
 self.left := 30;

 ButSetPos := TButton.Create(Self);
 ButSetPos.Parent := self;
 ButSetPos.Caption := 'Position setzen';
 ButSetPos.Left := 10;
 ButSetPos.Top := 10;
 ButSetPos.Width := 100;
 ButSetPos.Height := 30;
 ButSetPos.OnClick := @ButSetPosClick;

 LabelPos1 := TLabel.Create(self);
 LabelPos1.Parent := self;
 LabelPos1.Left := 120;
 LabelPos1.Top := 10;
 LabelPos1.Caption := 'X1 : 0' + #13#10 + 'Y1 : 0';

 LabelPos2 := TLabel.Create(self);
 LabelPos2.Parent := self;
 LabelPos2.Left := 200;
 LabelPos2.Top := 10;
 LabelPos2.Caption := 'X2 : 0' + #13#10 + 'Y2 : 0';

 Graph := TGraph2D.Create(self);
 Graph.Parent := self;
 Graph.Left := 10;
 Graph.Top := 50;
 Graph.AnchorParallel(akRight,10,self);
 Graph.AnchorParallel(akBottom,10,self);
 Graph.MinX :=   0;
 Graph.MaxX := 100;
 Graph.MaxY :=  100000;
 Graph.MinY := -100000;
 Graph.SetViewArea(0,-100,100,100);

 D3DDev9.CreateVertexBuffer(SizeOf(TVectorPointer_Vertex) * 4,D3DUSAGE_WRITEONLY,D3DFVF_XYZ or D3DFVF_DIFFUSE,D3DPOOL_MANAGED,Buf,nil);
 Buf.Lock(0,SizeOf(TVectorPointer_Vertex) * 5,PBuf,0);
 PBuffer(PBuf)^[0].X :=  Pos1.X / Sim.Width  * 2 - 1 - VecPointerWidth;
 PBuffer(PBuf)^[0].Y := -Pos1.Y / Sim.Height * 2 + 1 - VecPointerWidth;
 PBuffer(PBuf)^[0].Z := 0;
 PBuffer(PBuf)^[0].Color := $FFFFFF00;
 PBuffer(PBuf)^[1].X :=  Pos1.X / Sim.Width  * 2 - 1 - VecPointerWidth;
 PBuffer(PBuf)^[1].Y := -Pos1.Y / Sim.Height * 2 + 1 + VecPointerWidth;
 PBuffer(PBuf)^[1].Z := 0;
 PBuffer(PBuf)^[1].Color := $FFFFFF00;
 PBuffer(PBuf)^[2].X :=  Pos1.X / Sim.Width  * 2 - 1 + VecPointerWidth;
 PBuffer(PBuf)^[2].Y := -Pos1.Y / Sim.Height * 2 + 1 - VecPointerWidth;
 PBuffer(PBuf)^[2].Z := 0;
 PBuffer(PBuf)^[2].Color := $FFFFFF00;
 PBuffer(PBuf)^[3].X :=  Pos1.X / Sim.Width  * 2 - 1 + VecPointerWidth;
 PBuffer(PBuf)^[3].Y := -Pos1.Y / Sim.Height * 2 + 1 + VecPointerWidth;
 PBuffer(PBuf)^[3].Z := 0;
 PBuffer(PBuf)^[3].Color := $FFFFFF00;
 Buf.Unlock;

 SetLength(Graph.Lines,1);

 Show;
 ButSetPosClick(self);
end;

procedure T2DGraph.Render;
begin
 i := 0;
 while (i < Length(Graph.Lines[0])) do begin
  Graph.Lines[0][i].Y := 0;
  Graph.Lines[0][i].X := i / (Length(Graph.Lines[0])-1) * 100;
  inc(i);
 end;
 _j := 0;
 while (_j < Length(Graph.Lines[0])) do begin
  e := _j / (Length(Graph.Lines[0])-1);
  B := 0;
  while (B < 4) do begin
   Sim.CopyVectors(Round(Pos1.X * (1-e) + Pos2.X * e) + B mod 2,Round(Pos1.Y * (1-e) + Pos2.Y * e) + B div 2,Vec);
   i := 0;
   while (i < Length(Vec)) do begin
    Graph.Lines[0][_j].Y := Graph.Lines[0][_j].Y + Vec[i].Amp / 65536 * 100 * sin(Vec[i].Phi * 0.000095873);
    inc(i);
   end;
   inc(B);
  end;
  inc(_j);
 end;
 d3ddev9.SetStreamSource(0,Buf,0,SizeOf(TVectorPointer_Vertex));
 d3ddev9.SetFVF(D3DFVF_XYZ or D3DFVF_DIFFUSE);
 d3ddev9.DrawPrimitive(D3DPT_TRIANGLESTRIP,0,2);
 Graph.DoPaint(self);
end;

procedure T2DGraph.ButSetPosClick(Sender : TObject);
begin
 if (Is3DOn) then exit;
 Cont.Cursor := crCross;
 ControlSetMouseMove(Cont,@PanelMouseMove);
 ControlSetMouseDown(Cont,@PanelMouseDown);
 PosClick := 0;
end;

procedure T2DGraph.PanelMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
 Pos2.X := Round(X / Cont.Width * Sim.Width);
 Pos2.Y := Round(Y / Cont.Height * Sim.Height);
 if (PosClick = 0) then Pos1 := Pos2;
 LabelPos1.Caption := 'X1 : ' + IntToStr(Pos1.X) + #13#10 + 'Y1 : ' + IntToStr(Pos1.Y);
 LabelPos2.Caption := 'X2 : ' + IntToStr(Pos2.X) + #13#10 + 'Y2 : ' + IntToStr(Pos2.Y);
 e := arg(Pos2.X-Pos1.X,Pos1.Y-Pos2.Y);
 Buf.Lock(0,SizeOf(TVectorPointer_Vertex) * 5,PBuf,0);
 if (isNaN(e)) then begin
  PBuffer(PBuf)^[0].X :=  Pos1.X / Sim.Width  * 2 - 1 - VecPointerWidth;
  PBuffer(PBuf)^[0].Y := -Pos1.Y / Sim.Height * 2 + 1 - VecPointerWidth;
  PBuffer(PBuf)^[1].X :=  Pos1.X / Sim.Width  * 2 - 1 - VecPointerWidth;
  PBuffer(PBuf)^[1].Y := -Pos1.Y / Sim.Height * 2 + 1 + VecPointerWidth;
  PBuffer(PBuf)^[2].X :=  Pos1.X / Sim.Width  * 2 - 1 + VecPointerWidth;
  PBuffer(PBuf)^[2].Y := -Pos1.Y / Sim.Height * 2 + 1 - VecPointerWidth;
  PBuffer(PBuf)^[3].X :=  Pos1.X / Sim.Width  * 2 - 1 + VecPointerWidth;
  PBuffer(PBuf)^[3].Y := -Pos1.Y / Sim.Height * 2 + 1 + VecPointerWidth;
 end else begin
  PBuffer(PBuf)^[0].X :=  Pos1.X / Sim.Width  * 2 - 1 + cos(e+pi/2) * VecPointerWidth;
  PBuffer(PBuf)^[0].Y := -Pos1.Y / Sim.Height * 2 + 1 + sin(e+pi/2) * VecPointerWidth;
  PBuffer(PBuf)^[1].X :=  Pos2.X / Sim.Width  * 2 - 1 + cos(e+pi/2) * VecPointerWidth;
  PBuffer(PBuf)^[1].Y := -Pos2.Y / Sim.Height * 2 + 1 + sin(e+pi/2) * VecPointerWidth;
  PBuffer(PBuf)^[2].X :=  Pos1.X / Sim.Width  * 2 - 1 + cos(e-pi/2) * VecPointerWidth;
  PBuffer(PBuf)^[2].Y := -Pos1.Y / Sim.Height * 2 + 1 + sin(e-pi/2) * VecPointerWidth;
  PBuffer(PBuf)^[3].X :=  Pos2.X / Sim.Width  * 2 - 1 + cos(e-pi/2) * VecPointerWidth;
  PBuffer(PBuf)^[3].Y := -Pos2.Y / Sim.Height * 2 + 1 + sin(e-pi/2) * VecPointerWidth;
 end;
 Buf.Unlock;
 i := Round(sqrt((Pos2.X-Pos1.X)*(Pos2.X-Pos1.X) + (Pos2.Y-Pos1.Y)*(Pos2.Y-Pos1.Y)));
 if (i < 2) then i := 2;
 SetLength(Graph.Lines[0],i);
end;

procedure T2DGraph.PanelMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 if (PosClick = 0) then PosClick := 1
 else begin
  ControlSetMouseMove(Cont,nil);
  ControlSetMouseDown(Cont,nil);
  PanelMouseMove(Sender,Shift,X,Y);
  Cont.Cursor := crDefault;
 end;
end;

procedure T2DGraph.FormClose(Sender : TObject; var _Action : TCloseAction);
begin
 if (_OnClose <> nil) then _OnClose(self,_Action);
 _Action := caFree;
end;

destructor T2DGraph.Destroy;
var j : integer;
begin
 if (ControlGetMouseMove(Cont) = @PanelMouseMove) then ControlSetMouseMove(Cont,nil);
 if (ControlGetMouseDown(Cont) = @PanelMouseDown) then ControlSetMouseDown(Cont,nil);
 Buf := nil;
 inherited;
end;


end.

